@@ -23,9 +23,14 @@
 
 typedef struct SmlDevInfAgent SmlDevInfAgent;
 
+// only the SmlDevInf pointer of smlDevInfAgentNew is the local DevInf
+// all other SmlDevInf pointers are the DevInf of the remote peer
+
 SmlDevInfAgent *smlDevInfAgentNew(SmlDevInf *devinf, SmlError **error);
 void smlDevInfAgentFree(SmlDevInfAgent *agent);
+void smlDevInfAgentSetDevInf(SmlDevInfAgent *agent, SmlDevInf *devinf);
 SmlDevInf *smlDevInfAgentGetDevInf(SmlDevInfAgent *agent);
+SmlBool smlDevInfAgentRegisterSession(SmlDevInfAgent *agent, SmlManager *manager, SmlSession *session, SmlError **error);
 SmlBool smlDevInfAgentRegister(SmlDevInfAgent *agent, SmlManager *manager, SmlError **error);
 SmlBool smlDevInfAgentSendDevInf(SmlDevInfAgent *agent, SmlSession *session, SmlError **error);
 SmlBool smlDevInfAgentRequestDevInf(SmlDevInfAgent *agent, SmlSession *session, SmlError **error);
