@@ -220,6 +220,12 @@ error:
 	return NULL;
 }
 
+SmlDsServerType smlDsServerGetServerType(SmlDsServer *server)
+{
+	smlAssert(server);
+	return server->servertype;
+}
+
 void smlDsServerFree(SmlDsServer *server)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, server);
@@ -463,9 +469,10 @@ void smlDsSessionDispatch(SmlDsSession *dsession)
 	SmlStatus *reply = NULL;
 	
 	g_mutex_lock(dsession->lock);
+	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, dsession);
 
 	if (dsession->alertCommand && dsession->recvAlertCallback) {
-		smlTrace(TRACE_ENTRY, "%s(%p): Dispatching alert", __func__, dsession);
+		smlTrace(TRACE_INTERNAL, "%s: Dispatching alert", __func__);
 		
 		SmlErrorType type = SML_NO_ERROR;
 		if (!dsession->recvAlertCallback(dsession, dsession->alertCommand->private.alert.type, dsession->alertCommand->private.alert.anchor->last, dsession->alertCommand->private.alert.anchor->next, dsession->recvAlertCallbackUserdata))
@@ -485,9 +492,8 @@ void smlDsSessionDispatch(SmlDsSession *dsession)
 		smlCommandUnref(dsession->alertCommand);
 		dsession->alertCommand = NULL;
 
-		smlTrace(TRACE_EXIT, "%s", __func__);
 	} else if (dsession->recvSync && dsession->recvSyncCallback) {
-		smlTrace(TRACE_ENTRY, "%s(%p): Dispatching sync", __func__, dsession);
+		smlTrace(TRACE_INTERNAL, "%s: Dispatching sync", __func__);
 		
 		dsession->recvSyncCallback(dsession, ((SmlCommand *)(dsession->recvSync->data))->private.sync.numChanged, dsession->recvSyncCallbackUserdata);
 		dsession->recvSyncCallback = NULL;
@@ -519,7 +525,7 @@ void smlDsSessionDispatch(SmlDsSession *dsession)
 
 		smlTrace(TRACE_EXIT, "%s", __func__);
 	} else if (dsession->recvChanges && dsession->changesCallback) {
-		smlTrace(TRACE_ENTRY, "%s(%p): Dispatching changes", __func__, dsession);
+		smlTrace(TRACE_INTERNAL, "%s: Dispatching changes", __func__);
 		
 		while (dsession->recvChanges) {
 
@@ -569,17 +575,12 @@ void smlDsSessionDispatch(SmlDsSession *dsession)
 			dsession->recvEventCallback(dsession, SML_DS_EVENT_GOTCHANGES, dsession->recvEventCallbackUserdata);
 			smlTrace(TRACE_INTERNAL, "recvEventCallback all changes sent in recvChanges callback");
 		}
-
-		smlTrace(TRACE_EXIT, "%s", __func__);
 	} else {
-		smlTrace(TRACE_ENTRY, "%s()", __func__);
 		
 		smlTrace(TRACE_INTERNAL, "recvChanges: %p changesCallback: %p", dsession->recvChanges, dsession->changesCallback); 
-
-		smlTrace(TRACE_EXIT, "%s()", __func__);
-
 	}
 
+	smlTrace(TRACE_EXIT, "%s()", __func__);
 	g_mutex_unlock(dsession->lock);
 	
 	return;
