@@ -41,11 +41,15 @@
 
 static SmlBool _smlXmlAssemblerStartNodeNS(SmlXmlAssembler *assm, const char *name, const char *uri, SmlError **error)
 {
+	smlAssert(assm);
+	smlAssert(assm->writer);
+	smlAssert(name != NULL);
+	smlAssert(strlen(name) > 0);
 	smlTrace(TRACE_INTERNAL, "%s: Starting \"%s\"", __func__, name);
 	int rc = xmlTextWriterStartElementNS(assm->writer, NULL, (xmlChar *)name, (xmlChar *)uri);
 	if (rc < 0) {
-        smlErrorSet(error, SML_ERROR_GENERIC, "Unable to start node");
-	    return FALSE;
+        	smlErrorSet(error, SML_ERROR_GENERIC, "Unable to start node");
+		return FALSE;
 	}
 	return TRUE;
 }
@@ -57,9 +61,10 @@ static SmlBool _smlXmlAssemblerStartNode(SmlXmlAssembler *assm, const char *name
 
 static SmlBool _smlXmlAssemblerEndNode(SmlXmlAssembler *assm, SmlError **error)
 {
+	smlTrace(TRACE_INTERNAL, "%s: Ending node", __func__);
 	int rc = xmlTextWriterEndElement(assm->writer);
 	if (rc < 0) {
-		smlErrorSet(error, SML_ERROR_GENERIC, "Unable to end node");
+		smlErrorSet(error, SML_ERROR_GENERIC, "Unable to end node (%d).", rc);
 		return FALSE;
 	}
 	return TRUE;
@@ -69,7 +74,7 @@ static SmlBool _smlXmlAssemblerAddStringNS(SmlXmlAssembler *assm, const char *na
 {
 	int rc = xmlTextWriterWriteElementNS(assm->writer, NULL, (xmlChar *)name, (xmlChar *)uri, (xmlChar *)value);
 	if (rc < 0) {
-		smlErrorSet(error, SML_ERROR_GENERIC, "Unable to add string");
+		smlErrorSet(error, SML_ERROR_GENERIC, "Unable to add string (%d - %s:%s -> %s).", rc, uri, name, value);
 		return FALSE;
 	}
 	return TRUE;
@@ -77,7 +82,12 @@ static SmlBool _smlXmlAssemblerAddStringNS(SmlXmlAssembler *assm, const char *na
 
 static SmlBool _smlXmlAssemblerAddString(SmlXmlAssembler *assm, const char *name, const char *value, SmlError **error)
 {
-	return _smlXmlAssemblerAddStringNS(assm, name, NULL, value, error);
+	int rc = xmlTextWriterWriteElement(assm->writer, (xmlChar *)name, (xmlChar *)value);
+	if (rc < 0) {
+		smlErrorSet(error, SML_ERROR_GENERIC, "Unable to add pure string (%d - %s -> %s).", rc, name, value);
+		return FALSE;
+	}
+	return TRUE;
 }
 
 static SmlBool _smlXmlAssemblerAddData(SmlXmlAssembler *assm, const char *name, const char *value, unsigned int size, SmlBool raw, SmlError **error)
@@ -92,7 +102,7 @@ static SmlBool _smlXmlAssemblerAddData(SmlXmlAssembler *assm, const char *name,
 		rc = xmlTextWriterWriteFormatCDATA(assm->writer, "%*s", size, (xmlChar *)value);
 		
 	if (rc < 0) {
-		smlErrorSet(error, SML_ERROR_GENERIC, "Unable to add data");
+		smlErrorSet(error, SML_ERROR_GENERIC, "Unable to add data (%d).", rc);
 		return FALSE;
 	}
 		
@@ -175,12 +185,13 @@ SmlBool smlAnchorAssemble(SmlAnchor *anchor, SmlXmlAssembler *assm, SmlError **e
 		goto error;
 	}
 	
+	/* SyncML Meta-Information DTD 1.1, Page 6, 5.1 Anchor - Content Model: (Last?, Next)
+	   Last is optional, don't write a empty Last node.
+	 */ 
 	if (anchor->last) {
 		if (!_smlXmlAssemblerAddStringNS(assm, SML_ELEMENT_LAST, SML_NAMESPACE_METINF, anchor->last, error))
 			goto error;
-	} else {
-		if (!_smlXmlAssemblerAddStringNS(assm, SML_ELEMENT_LAST, SML_NAMESPACE_METINF, "", error))
-			goto error;
+		
 	}
 	
 	if (!_smlXmlAssemblerAddStringNS(assm, SML_ELEMENT_NEXT, SML_NAMESPACE_METINF, anchor->next, error))
@@ -271,8 +282,9 @@ SmlBool smlCredAssemble(SmlCred *cred, SmlXmlAssembler *assm, SmlError **error)
 		case SML_FORMAT_TYPE_BASE64:
 			if (!_smlXmlAssemblerAddStringNS(assm, SML_ELEMENT_FORMAT, SML_NAMESPACE_METINF, SML_BASE64, error))
 				goto error;
+			break;
 		default:
-			smlErrorSet(error, SML_ERROR_GENERIC, "Unknown format");
+			smlErrorSet(error, SML_ERROR_GENERIC, "SyncML credential: Unknown format %d.", cred->format);
 			goto error;
 	}
 	
@@ -280,9 +292,11 @@ SmlBool smlCredAssemble(SmlCred *cred, SmlXmlAssembler *assm, SmlError **error)
 		case SML_AUTH_TYPE_BASIC:
 			if (!_smlXmlAssemblerAddStringNS(assm, SML_ELEMENT_TYPE, SML_NAMESPACE_METINF, SML_AUTH_BASIC, error))
 				goto error;
+			break;
 		case SML_AUTH_TYPE_MD5:
 			if (!_smlXmlAssemblerAddStringNS(assm, SML_ELEMENT_TYPE, SML_NAMESPACE_METINF, SML_AUTH_MD5, error))
 				goto error;
+			break;
 		default:
 			smlErrorSet(error, SML_ERROR_GENERIC, "Unknown format");
 			goto error;
@@ -425,18 +439,19 @@ SmlBool smlSyncAssemble(SmlXmlAssembler *assm, SmlCommand *cmd, SmlError **error
 		if (smlAssemblerGetRequestedMaxObjSize(assm->assembler) == -1)
 			supportsLargeObjects = FALSE;
 	}
-	
-	if (supportsLargeObjects && smlSessionGetReceivingMaxObjSize(assm->session) && assm->session->version != SML_VERSION_10) {
-		if (!_smlXmlAssemblerStartNode(assm, SML_ELEMENT_META, error))
-			goto error;
-		
-		if (!_smlXmlAssemblerAddIDNS(assm, SML_ELEMENT_MAXOBJSIZE, SML_NAMESPACE_METINF, smlSessionGetReceivingMaxObjSize(assm->session), error))
-			goto error;
-		
-		//META
-		if (!_smlXmlAssemblerEndNode(assm, error))
-			goto error;
-	}
+
+	// this is done by the session
+	//if (supportsLargeObjects && smlSessionGetReceivingMaxObjSize(assm->session) && assm->session->version != SML_VERSION_10) {
+	//	if (!_smlXmlAssemblerStartNode(assm, SML_ELEMENT_META, error))
+	//		goto error;
+	//	
+	//	if (!_smlXmlAssemblerAddIDNS(assm, SML_ELEMENT_MAXOBJSIZE, SML_NAMESPACE_METINF, smlSessionGetReceivingMaxObjSize(assm->session), error))
+	//		goto error;
+	//	
+	//	//META
+	//	if (!_smlXmlAssemblerEndNode(assm, error))
+	//		goto error;
+	//}
 	
 	if (!smlLocationAssemble(cmd->target, assm, SML_ELEMENT_TARGET, error))
 		goto error;
@@ -549,14 +564,17 @@ SmlBool smlAlertAssemble(SmlXmlAssembler *assm, SmlCommand *cmd, SmlError **erro
 			goto error;
 	}
 	
-	if (!cmd->source) {
-		smlErrorSet(error, SML_ERROR_GENERIC, "No source set");
-		goto error;
+	if (cmd->source) {
+		if (!smlLocationAssemble(cmd->source, assm, SML_ELEMENT_SOURCE, error))
+			goto error;
+	} else {
+		// NEXT MESSAGE alerts does not need a source/target
+		if (cmd->private.alert.type != SML_ALERT_NEXT_MESSAGE) {
+			smlErrorSet(error, SML_ERROR_GENERIC, "No source set");
+			goto error;
+		}
 	}
 	
-	if (!smlLocationAssemble(cmd->source, assm, SML_ELEMENT_SOURCE, error))
-		goto error;
-	
 	if (cmd->private.alert.anchor) {
 		if (!_smlXmlAssemblerStartNode(assm, SML_ELEMENT_META, error))
 			goto error;
@@ -640,6 +658,7 @@ SmlBool smlXmlAssemblerAddHeader(SmlXmlAssembler *assm, SmlSession *session, Sml
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, assm, session, error);
 	smlAssert(assm);
+	smlAssert(g_mutex_trylock(assm->mutex));
 	smlAssert(session);
 	
 	/* Lets see if there already was a header before */
@@ -716,12 +735,17 @@ SmlBool smlXmlAssemblerAddHeader(SmlXmlAssembler *assm, SmlSession *session, Sml
 	if (!_smlXmlAssemblerAddID(assm, SML_ELEMENT_MSGID, session->lastMessageID, error))
 		goto error_free_writer;
 	
-	if (!smlLocationAssemble(session->source, assm, SML_ELEMENT_TARGET, error))
+	if (!smlLocationAssemble(session->target, assm, SML_ELEMENT_TARGET, error))
 		goto error_free_writer;
 	
-	if (!smlLocationAssemble(session->target, assm, SML_ELEMENT_SOURCE, error))
+	if (!smlLocationAssemble(session->source, assm, SML_ELEMENT_SOURCE, error))
 		goto error_free_writer;
 	
+	if (session->cred != NULL) {
+		if (!smlCredAssemble(session->cred, assm, error))
+			goto error_free_writer;
+	}
+	
 	//const char *opt = smlAssemblerGetOption(assm->assembler, "USE_LARGEOBJECTS");
 	//SmlBool supportsLargeObjects = (opt && !atoi(opt)) ? FALSE : TRUE;
 	if (session->incomingMaxMsgSize) { // || (session->incomingMaxObjSize && supportsLargeObjects)) {
@@ -732,6 +756,10 @@ SmlBool smlXmlAssemblerAddHeader(SmlXmlAssembler *assm, SmlSession *session, Sml
 			if (!_smlXmlAssemblerAddIDNS(assm, SML_ELEMENT_MAXMSGSIZE, SML_NAMESPACE_METINF, session->incomingMaxMsgSize, error))
 				goto error_free_writer;
 		}
+		if (session->receivingMaxObjSize) {
+			if (!_smlXmlAssemblerAddIDNS(assm, SML_ELEMENT_MAXOBJSIZE, SML_NAMESPACE_METINF, session->receivingMaxObjSize, error))
+				goto error_free_writer;
+		}
 		
 		/*if (supportsLargeObjects && session->incomingMaxObjSize) {
 			if (!_smlXmlAssemblerAddIDNS(assm, SML_ELEMENT_MAXOBJSIZE, SML_NAMESPACE_METINF, session->incomingMaxObjSize, error))
@@ -755,6 +783,7 @@ SmlBool smlXmlAssemblerAddHeader(SmlXmlAssembler *assm, SmlSession *session, Sml
 	xmlFreeTextWriter(assm->writer);
 	assm->writer = NULL;
 
+	g_mutex_unlock(assm->mutex);
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
@@ -765,6 +794,7 @@ error_free_buffer:
 	xmlBufferFree(assm->header_buffer);
 	assm->header_buffer = NULL;
 error:
+	g_mutex_unlock(assm->mutex);
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return FALSE;
 }
@@ -1030,6 +1060,7 @@ SmlBool smlXmlAssemblerAddStatus(SmlXmlAssembler *assm, SmlStatus *status, SmlEr
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, assm, status, error);
 	smlAssert(assm);
+	smlAssert(g_mutex_trylock(assm->mutex));
 	smlAssert(status);
 	SmlXmlAssemblerStatus *res = NULL;
 	
@@ -1195,6 +1226,7 @@ SmlBool smlXmlAssemblerAddStatus(SmlXmlAssembler *assm, SmlStatus *status, SmlEr
 	
 	assm->added_statuses++;
 	
+	g_mutex_unlock(assm->mutex);
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 	
@@ -1204,6 +1236,7 @@ error_free_writer:
 error_free_buffer:
 	xmlBufferFree(res->buffer);
 error:
+	g_mutex_unlock(assm->mutex);
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return FALSE;
 }
@@ -1253,7 +1286,8 @@ void smlXmlAssemblerFree(SmlXmlAssembler *assm)
 	}
 	
 	flush_list(assm->commands);
-	
+
+	g_mutex_free(assm->mutex);
 	g_free(assm);
 
 	smlTrace(TRACE_EXIT, "%s", __func__);
@@ -1278,13 +1312,14 @@ unsigned int smlXmlAssemblerFlush(SmlXmlAssembler *assm)
 		if (!status->buffer) {
 			/* If the status was not yet added, we have to reset the command id.
 			 * We start with id #2 (since #1 will be the header reply) */
+			/* FIXME: why should we reset a command id? */
 			newid++;
-			status->cmdID = newid;
+			// status->cmdID = newid;
 			missing = TRUE;
 		} else if (!missing) {
 			xmlBufferFree(status->buffer);
-			g_free(status);
 			assm->statuses = g_list_remove(assm->statuses, status);
+			g_free(status);
 			assm->reserved_statuses--;
 			assm->added_statuses--;
 		}
@@ -1305,10 +1340,12 @@ SmlBool smlXmlAssemblerStart(SmlXmlAssembler *assm, SmlSession *session, SmlErro
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, assm, session, error);
 	smlAssert(assm);
+	smlAssert(g_mutex_trylock(assm->mutex));
 	smlAssert(session);
 
 	assm->session = session;
 	
+	g_mutex_unlock(assm->mutex);
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 }
@@ -1318,6 +1355,7 @@ SmlBool smlXmlAssemblerEnd(SmlXmlAssembler *assm, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p)", __func__, assm, error);
 	smlAssert(assm);
+	smlAssert(g_mutex_trylock(assm->mutex));
 	
 	//Close syncml
 	if (!_smlXmlAssemblerEndNode(assm, error))
@@ -1328,16 +1366,19 @@ SmlBool smlXmlAssemblerEnd(SmlXmlAssembler *assm, SmlError **error)
 		goto error;
 	}
 		
+	g_mutex_unlock(assm->mutex);
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
 error:
+	g_mutex_unlock(assm->mutex);
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return FALSE;
 }
 
 SmlBool smlXmlAssemblerAddChildren(SmlXmlAssembler *assm, GList *b, SmlError **error)
 {
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, assm, b, error);
 	smlAssert(assm);
 	const char *opt = smlAssemblerGetOption(assm->assembler, "ONLY_REPLACE");
 	SmlBool onlyReplace = (opt && atoi(opt)) ? TRUE : FALSE;
@@ -1362,8 +1403,12 @@ SmlBool smlXmlAssemblerAddChildren(SmlXmlAssembler *assm, GList *b, SmlError **e
 				
 				if (!_smlXmlAssemblerStartNode(assm, cmdname, error))
 					goto error;
-					
-				xmlTextWriterWriteRawLen(assm->writer, xmlBufferContent(cmd->buffer), xmlBufferLength(cmd->buffer) - 1);
+
+				int rc = xmlTextWriterWriteRawLen(assm->writer, xmlBufferContent(cmd->buffer), xmlBufferLength(cmd->buffer));
+				if (rc < 0) {
+					smlErrorSet(error, SML_ERROR_GENERIC, "Unable to write raw node data (%d).", rc);
+					goto error;
+				}
 				
 				if (cmd->children) {
 					if (!smlXmlAssemblerAddChildren(assm, cmd->children, error))
@@ -1381,11 +1426,12 @@ SmlBool smlXmlAssemblerAddChildren(SmlXmlAssembler *assm, GList *b, SmlError **e
 				break;
 		}
 	}
-	
+
+	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
 error:
-	smlTrace(TRACE_ERROR, "%s: %s", __func__, smlErrorPrint(error));
+	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return FALSE;
 }
 
@@ -1393,6 +1439,7 @@ SmlBool smlXmlAssemblerRunFull(SmlXmlAssembler *assm, char **data, unsigned int
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p, %i, %i, %i, %p)", __func__, assm, data, size, end, final, check, maxsize, error);
 	smlAssert(assm);
+	smlAssert(g_mutex_trylock(assm->mutex));
 	smlAssert(data);
 	smlAssert(size);
 	smlAssert(assm->session);
@@ -1461,34 +1508,48 @@ SmlBool smlXmlAssemblerRunFull(SmlXmlAssembler *assm, char **data, unsigned int
 	}
 	
 	/* Add the header */
-	xmlTextWriterWriteRawLen(assm->writer, xmlBufferContent(assm->header_buffer), xmlBufferLength(assm->header_buffer) - 1);
+	int rc = xmlTextWriterWriteRawLen(assm->writer, xmlBufferContent(assm->header_buffer), xmlBufferLength(assm->header_buffer));
+	if (rc < 0) {
+		smlErrorSet(error, SML_ERROR_GENERIC, "Unable to write raw header data (%d).", rc);
+		goto error_free_writer;
+	}
 	
 	/* Start the sync body */
 	if (!_smlXmlAssemblerStartNode(assm, SML_ELEMENT_SYNCBODY, error))
 		goto error_free_writer;
-	
-	buffersize += 20 + (xmlBufferLength(assm->header_buffer) - 1);
+
+	// FIXME: why do we need 20 extra bytes here?
+	// FIXME: buffersize is never read !!!
+	buffersize += 20 + xmlBufferLength(assm->header_buffer);
 	
 	/* Add the statuses */
 	GList *b = NULL;
 	SmlBool missingstatus = FALSE;
-	smlTrace(TRACE_INTERNAL, "Now adding %i statuses", g_list_length(assm->statuses));
+	smlTrace(TRACE_INTERNAL, "%s: Now adding %i statuses",
+		__func__, g_list_length(assm->statuses));
 	for (b = assm->statuses; b; b = b->next) {
 		SmlXmlAssemblerStatus *status = b->data;
-		if (!status->buffer) {
+		if (!status->buffer || xmlBufferLength(status->buffer) == 0) {
 			if (status->cmdRef == 0 && check) {
 				smlErrorSet(error, SML_ERROR_GENERIC, "Reserved status 0 has not been added");
 				goto error_free_writer;
 			}
-			smlTrace(TRACE_INTERNAL, "Reserved status %i is missing", status->cmdRef);
+			smlTrace(TRACE_INTERNAL, "%s: Reserved status %i is missing",
+				__func__, status->cmdRef);
 			missingstatus = TRUE;
 			break;
 		}
-			
-		buffersize += xmlBufferLength(status->buffer) - 1;
+
+		buffersize += xmlBufferLength(status->buffer);
 		//if (maxsize && buffersize > maxsize)
 		//	break;
-		xmlTextWriterWriteRawLen(assm->writer, xmlBufferContent(status->buffer), xmlBufferLength(status->buffer) - 1);
+		rc = xmlTextWriterWriteRawLen(assm->writer, xmlBufferContent(status->buffer), xmlBufferLength(status->buffer));
+		if (rc < 0) {
+			smlErrorSet(error, SML_ERROR_GENERIC, "Unable to write raw status data (%d - %d of %d: %s).",
+				rc, xmlBufferLength(status->buffer),
+				buffersize, xmlBufferContent(status->buffer));
+			goto error_free_writer;
+		}
 	}
 	
 	/* We cannot have missing statuses when its final. And we cannot add commands */
@@ -1502,6 +1563,7 @@ SmlBool smlXmlAssemblerRunFull(SmlXmlAssembler *assm, char **data, unsigned int
 		goto error_free_writer;
 	
 	if (final) {
+		smlTrace(TRACE_INTERNAL, "%s: setting final", __func__);
 		if (!_smlXmlAssemblerAddString(assm, SML_ELEMENT_FINAL, "", error))
 			goto error_free_writer;
 	}
@@ -1514,7 +1576,7 @@ SmlBool smlXmlAssemblerRunFull(SmlXmlAssembler *assm, char **data, unsigned int
 	if (!_smlXmlAssemblerEndNode(assm, error))
 		goto error_free_writer;
 	
-    if (xmlTextWriterEndDocument(assm->writer) < 0) {
+	if (xmlTextWriterEndDocument(assm->writer) < 0) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "Unable to end writer");
 		goto error;
 	}
@@ -1522,24 +1584,22 @@ SmlBool smlXmlAssemblerRunFull(SmlXmlAssembler *assm, char **data, unsigned int
 	
 	xmlFreeTextWriter(assm->writer);
 	assm->writer = NULL;
-	
-	//Avoid the memory copy here
-	*data = (char *)xmlBufferContent(buffer);
-	*size = xmlBufferLength(buffer) - 1;
+
+	// the buffer was created with xmlBufferCreateSize
+	// so we must free it via the libxml API and not via g_free
+	*size = xmlBufferLength(buffer);
+	*data = g_strndup(xmlBufferContent(buffer), *size);
+	xmlBufferFree(buffer);
 	if (end) {
-		if (final && !assm->commands)
-			*end = TRUE;
-		else
-			*end = FALSE;
+	 	if (final && !assm->commands)
+	 		*end = TRUE;
+	 	else
+	 		*end = FALSE;
 	}
-	
-	g_free(buffer);
-	
-	//FIXME ADD debug ifs
-	char *debugstr = smlPrintBinary(*data, *size);
-	smlTrace(TRACE_INTERNAL, "Message Assembled: %s", debugstr);
-	g_free(debugstr);
-	
+	smlTrace(TRACE_INTERNAL, "%s: Message Assembled: %s",
+		__func__, *data);
+
+	g_mutex_unlock(assm->mutex);
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
@@ -1549,6 +1609,7 @@ error_free_writer:
 error_free_buffer:
 	xmlBufferFree(buffer);
 error:
+	g_mutex_unlock(assm->mutex);
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return FALSE;
 }
@@ -1644,6 +1705,8 @@ SmlXmlAssembler *smlXmlAssemblerNew(SmlAssembler *assembler, SmlAssemblerFunctio
 	if (!assm)
 		goto error;
 	assm->assembler = assembler;
+	assm->mutex = g_mutex_new();
+	if (!assm->mutex) goto error;
 	
 	functions->start = (SmlAssemblerStartFunction)smlXmlAssemblerStart;
 	functions->free = (SmlAssemblerFreeFunction)smlXmlAssemblerFree;
@@ -1663,7 +1726,10 @@ SmlXmlAssembler *smlXmlAssemblerNew(SmlAssembler *assembler, SmlAssemblerFunctio
 	
 	smlTrace(TRACE_EXIT, "%s: %p", __func__, assm);
 	return assm;
-	
+
+error_mutex:
+	g_free(assm);
+	smlErrorSet(error, SML_ERROR_GENERIC, "%s - Cannot create new mutex.", __func__);
 error:
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return NULL;
@@ -1696,11 +1762,174 @@ error:
 	return FALSE;
 }
 
-static SmlBool _smlXmlDevInfDataStoreAssemble(SmlXmlAssembler *assm, SmlDevInfDataStore *datastore, SmlError **error)
+static SmlBool _smlXmlDevInfDataStoreAssembleCTCap(
+			SmlXmlAssembler *assm,
+			SmlDevInfCTCap *ctcap,
+			SmlBool flat,
+			SmlError **error)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, assm, datastore, error);
+	smlTrace(TRACE_ENTRY, "%s(%p, %p)", __func__, assm, ctcap, error);
+	smlAssert(assm);
+	smlAssert(ctcap);
+
+	if (!flat)
+	{
+		// start CTCap
+		if (!_smlXmlAssemblerStartNode(assm, SML_ELEMENT_CTCAP, error))
+			goto error;
+	}
+
+	// add CTType
+	if (!_smlXmlAssemblerAddString(assm, SML_ELEMENT_CTTYPE, ctcap->ct->cttype, error))
+		goto error;
+
+	// add VerCT
+	if (!_smlXmlAssemblerAddString(assm, SML_ELEMENT_VERCT, ctcap->ct->verct, error))
+		goto error;
+
+	// add properties
+	GList *hprop = NULL;
+	for (hprop = ctcap->properties; hprop; hprop = hprop->next)
+	{
+		SmlDevInfProperty *property = hprop->data;
+
+		if (!flat)
+		{
+			// add Property
+			if (!_smlXmlAssemblerStartNode(assm, SML_ELEMENT_PROPERTY, error))
+				goto error;
+		}
+
+		// add PropName
+		if (property->propName != NULL &&
+		    !_smlXmlAssemblerAddString(assm, SML_ELEMENT_PROPNAME, property->propName, error))
+			goto error;
+
+		// add DataType
+		if (property->dataType != NULL &&
+		    !_smlXmlAssemblerAddString(assm, SML_ELEMENT_DATATYPE, property->dataType, error))
+			goto error;
+
+		// add MaxOccur
+		if (property->maxOccur > 0 &&
+		    !_smlXmlAssemblerAddID(assm, SML_ELEMENT_MAXOCCUR, property->maxOccur, error))
+			goto error;
+
+		// add MaxSize
+		if (property->maxSize > 0 &&
+		    !_smlXmlAssemblerAddID(assm, SML_ELEMENT_MAXSIZE, property->maxSize, error))
+			goto error;
+
+		// add NoTruncate
+		if (property->noTruncate)
+		{
+			if (!_smlXmlAssemblerStartNode(assm, SML_ELEMENT_NOTRUNCATE, error))
+				goto error;
+			if (!_smlXmlAssemblerEndNode(assm, error))
+				goto error;
+		}
+
+		// add DisplayName
+		if (property->displayName != NULL &&
+		    !_smlXmlAssemblerAddString(assm, SML_ELEMENT_DISPLAYNAME, property->displayName, error))
+			goto error;
+
+		// add values
+		GList *hvalue = NULL;
+		for (hvalue = property->valEnums; hvalue; hvalue = hvalue->next)
+		{
+			char *valEnum = hvalue->data;
+
+			// add ValEnum
+		    	if (!_smlXmlAssemblerAddString(assm, SML_ELEMENT_VALENUM, valEnum, error))
+				goto error;
+			
+		} // end values for loop
+
+		// add parameters
+		GList *hparam = NULL;
+		for (hparam = property->propParams; hparam; hparam = hparam->next)
+		{
+			SmlDevInfPropParam *param = hparam->data;
+
+			if (!flat)
+			{
+				// add PropParam
+				if (!_smlXmlAssemblerStartNode(assm, SML_ELEMENT_PROPPARAM, error))
+					goto error;
+			}
+
+			// add ParamName
+			if (param->paramName != NULL &&
+			    !_smlXmlAssemblerAddString(assm, SML_ELEMENT_PARAMNAME, param->paramName, error))
+				goto error;
+
+			// add DataType
+			if (param->dataType != NULL &&
+			    !_smlXmlAssemblerAddString(assm, SML_ELEMENT_DATATYPE, param->dataType, error))
+				goto error;
+
+			// add DisplayName
+			if (param->displayName != NULL &&
+		    	    !_smlXmlAssemblerAddString(assm, SML_ELEMENT_DISPLAYNAME, param->displayName, error))
+				goto error;
+
+			// add values
+			GList *hvalue = NULL;
+			for (hvalue = param->valEnums; hvalue; hvalue = hvalue->next)
+			{
+				char *valEnum = hvalue->data;
+
+				// add ValEnum
+		    		if (!_smlXmlAssemblerAddString(assm, SML_ELEMENT_VALENUM, valEnum, error))
+					goto error;
+			
+			} // end values for loop
+
+			if (!flat)
+			{
+				// end PropParam
+				if (!_smlXmlAssemblerEndNode(assm, error))
+					goto error;
+			}
+
+		} // end params for loop
+
+		if (!flat)
+		{
+			// end Property
+			if (!_smlXmlAssemblerEndNode(assm, error))
+				goto error;
+		}
+
+	} //end of ctcap->properties for loop
+
+	if (!flat)
+	{
+		// end CTCap
+		if (!_smlXmlAssemblerEndNode(assm, error))
+			goto error;
+	}
+
+	smlTrace(TRACE_EXIT, "%s", __func__);
+	return TRUE;
+	
+error:
+	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
+	return FALSE;
+}
+
+static SmlBool _smlXmlDevInfDataStoreAssemble(
+			SmlXmlAssembler *assm,
+			SmlDevInfDataStore *datastore,
+			SmlDevInf *devinf,
+			SmlError **error)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p, %p)", __func__, assm, datastore, devinf, error);
 	smlAssert(datastore);
 	smlAssert(assm);
+
+	GList *contentTypes = NULL;
 	
 	if (!datastore->sourceref) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "Missing sourceref");
@@ -1745,22 +1974,67 @@ static SmlBool _smlXmlDevInfDataStoreAssemble(SmlXmlAssembler *assm, SmlDevInfDa
 	//rx-pref
 	if (!_smlXmlDevInfDataStoreAssembleRxTx(assm, SML_ELEMENT_RXPREF, datastore->rxPrefContentType, datastore->rxPrefVersion, error))
 		goto error;
+	contentTypes = g_list_append(
+				contentTypes,
+				smlDevInfNewContentType(
+					datastore->rxPrefContentType,
+					datastore->rxPrefVersion, error));
 			
 	//rx
 	if (datastore->rxContentType) {
 		if (!_smlXmlDevInfDataStoreAssembleRxTx(assm, SML_ELEMENT_RX, datastore->rxContentType, datastore->rxVersion, error))
 			goto error;
+		contentTypes = g_list_append(
+				contentTypes,
+				smlDevInfNewContentType(
+					datastore->rxContentType,
+					datastore->rxVersion, error));
 	}
 	
 	//tx-pref
 	if (!_smlXmlDevInfDataStoreAssembleRxTx(assm, SML_ELEMENT_TXPREF, datastore->txPrefContentType, datastore->txPrefVersion, error))
 		goto error;
+	contentTypes = g_list_append(
+				contentTypes,
+				smlDevInfNewContentType(
+					datastore->txPrefContentType,
+					datastore->txPrefVersion, error));
 			
 	//tx
 	if (datastore->txContentType) {
 		if (!_smlXmlDevInfDataStoreAssembleRxTx(assm, SML_ELEMENT_TX, datastore->txContentType, datastore->txVersion, error))
 			goto error;
+		contentTypes = g_list_append(
+				contentTypes,
+				smlDevInfNewContentType(
+					datastore->txContentType,
+					datastore->txVersion, error));
 	}
+
+	// CTCap (if SyncML version 1.2 device info)
+	if (devinf->version >= SML_DEVINF_VERSION_12)
+	{
+		GList *hct = NULL;
+		for (hct = contentTypes; hct; hct = hct->next) {
+			SmlDevInfContentType *ct = hct->data;
+			SmlDevInfCTCap *ctcap = smlDevInfGetCTCap(devinf, ct);
+			if (ctcap != NULL)
+			{
+				// we found a matching CTCap
+				// so let's dump it
+				if (!_smlXmlDevInfDataStoreAssembleCTCap(assm, ctcap, FALSE, error))
+					goto error;
+			} else {
+				// we have a content type without CTCap
+				// this is a real source of trouble
+				// WARNING: should we fail on this issue?
+				smlTrace(TRACE_INTERNAL, "%s: found a content type (%s %d) without CTCap",
+					 __func__, ct->cttype, ct->verct);
+			}
+		} // end of contentTypes for loop
+	} // end of SML_DEVINF_VERSION_12
+	g_list_free(contentTypes);
+		
 	
 	//Dsmem
 	if (datastore->maxmem || datastore->maxid) {
@@ -1847,6 +2121,15 @@ SmlBool smlXmlDevInfAssemble(SmlDevInf *devinf, SmlDevInfVersion version, char *
 	smlAssert(devinf);
 	smlAssert(data);
 	smlAssert(size);
+
+	// sometime devinf->version is empty
+	// both version fields should be identical
+	smlTrace(TRACE_INTERNAL, "devinf version: %i, version: %i", devinf->version, version);
+	if (devinf->version == SML_DEVINF_VERSION_UNKNOWN)
+		devinf->version = version;
+	if (version == SML_DEVINF_VERSION_UNKNOWN)
+		version = devinf->version;
+	smlAssert(devinf->version == version);
 	
 	SmlXmlAssembler *assm = smlTryMalloc0(sizeof(SmlXmlAssembler), error);
 	if (!assm)
@@ -1871,6 +2154,10 @@ SmlBool smlXmlDevInfAssemble(SmlDevInf *devinf, SmlDevInfVersion version, char *
 	
 	//Verdtd
 	switch (version) {
+		case SML_DEVINF_VERSION_12:
+			if (!_smlXmlAssemblerAddString(assm, SML_ELEMENT_VERDTD, "1.2", error))
+				goto error_free_writer;
+			break;
 		case SML_DEVINF_VERSION_11:
 			if (!_smlXmlAssemblerAddString(assm, SML_ELEMENT_VERDTD, "1.1", error))
 				goto error_free_writer;
@@ -1955,21 +2242,21 @@ SmlBool smlXmlDevInfAssemble(SmlDevInf *devinf, SmlDevInfVersion version, char *
 	GList *d = NULL;
 	for (d = devinf->datastores; d; d = d->next) {
 		SmlDevInfDataStore *datastore = d->data;
-		if (!_smlXmlDevInfDataStoreAssemble(assm, datastore, error))
+		if (!_smlXmlDevInfDataStoreAssemble(assm, datastore, devinf, error))
 			goto error_free_buffer;
 	}
 
-	// CTCap
-	if (devinf->ctcaps) {
+	// CTCap (only for SyncML 1.0 and 1.1)
+	if (
+	    devinf->version < SML_DEVINF_VERSION_12 &&
+	    devinf->ctcaps) {
 		if (!_smlXmlAssemblerStartNode(assm, SML_ELEMENT_CTCAP, error))
 			goto error;
 		
 		for (d = devinf->ctcaps; d; d = d->next) {
 			SmlDevInfCTCap *ctcap = d->data;
-			const char *element_name;
-			if (!(element_name = smlDevInfCTCapTypeToString(ctcap->type, error)))
-				goto error;
-			if (!_smlXmlAssemblerAddString(assm, element_name, ctcap->value, error))
+
+			if (!_smlXmlDevInfDataStoreAssembleCTCap(assm, ctcap, TRUE, error))
 				goto error;
 		}
 		
@@ -1990,9 +2277,9 @@ SmlBool smlXmlDevInfAssemble(SmlDevInf *devinf, SmlDevInfVersion version, char *
 	xmlFreeTextWriter(assm->writer);
 	assm->writer = NULL;
 	
-	*data = g_strdup((char *)xmlBufferContent(buffer));
-	*size = xmlBufferLength(buffer) - 1;
-	
+	*size = xmlBufferLength(buffer);
+	*data = g_strndup(xmlBufferContent(buffer), *size);
+
 	xmlBufferFree(buffer);
 	
 	g_free(assm);
