@@ -51,6 +51,17 @@ static SmlBool _smlXmlParserStep(SmlXmlParser *parser)
 	return ret;
 }
 
+static SmlBool _smlXmlParserStepData(SmlXmlParser *parser)
+{
+	SmlBool ret = FALSE;
+	do {
+		ret = (xmlTextReaderRead(parser->reader) == 1) ? TRUE : FALSE;
+	} while (ret && (xmlTextReaderNodeType(parser->reader) == XML_READER_TYPE_DOCUMENT_TYPE || \
+		xmlTextReaderNodeType(parser->reader) == XML_READER_TYPE_WHITESPACE));
+		
+	return ret;
+}
+
 static SmlBool _smlXmlParserExpectNode(SmlXmlParser *parser, int type, SmlBool empty, const char *name, SmlError **error)
 {
 	if (!_smlXmlParserStep(parser)) {
@@ -181,9 +192,13 @@ static SmlBool _smlXmlParserGetData(SmlXmlParser *parser, char **string, unsigne
 		smlErrorSet(error, SML_ERROR_GENERIC, "Unable to create new writer");
 		goto error_free_buffer;
 	}
+
+	/* SyncML 1.2 allows empty data elements <Data /> */
+	if (xmlTextReaderIsEmptyElement(parser->reader))
+		goto out;
 		
 	while (1) {
-		if (!_smlXmlParserStep(parser)) {
+		if (!_smlXmlParserStepData(parser)) {
 			smlErrorSet(error, SML_ERROR_GENERIC, "Missing nodes");
 			goto error_free_writer;
 		}
@@ -193,7 +208,9 @@ static SmlBool _smlXmlParserGetData(SmlXmlParser *parser, char **string, unsigne
 				if (!strcmp((char *)xmlTextReaderConstName(parser->reader), name))
 					goto out;
 					
-				rc = xmlTextWriterEndElement(writer);
+				if (!xmlTextReaderIsEmptyElement(parser->reader))
+					rc = xmlTextWriterEndElement(writer);
+
 				if (rc < 0) {
 					smlErrorSet(error, SML_ERROR_GENERIC, "Unable to end node");
 					goto error_free_writer;
@@ -218,6 +235,13 @@ static SmlBool _smlXmlParserGetData(SmlXmlParser *parser, char **string, unsigne
 					goto error_free_writer;
 				}
 				break;
+			case XML_READER_TYPE_SIGNIFICANT_WHITESPACE:
+				rc = xmlTextWriterWriteRaw(writer, xmlTextReaderConstValue(parser->reader));
+				if (rc < 0) {
+					smlErrorSet(error, SML_ERROR_GENERIC, "Unable to add string");
+					goto error_free_writer;
+				}
+				break;
 			default:
 				smlErrorSet(error, SML_ERROR_GENERIC, "Unknown node type");
 				goto error_free_writer;
@@ -225,19 +249,19 @@ static SmlBool _smlXmlParserGetData(SmlXmlParser *parser, char **string, unsigne
 	}
 
 out:
-	if (xmlTextWriterEndDocument(writer) < 0) {
-		smlErrorSet(error, SML_ERROR_GENERIC, "Unable to end writer");
-		goto error_free_writer;
-	}
-	
+	// do not call xmlTextWriterEndDocument here
+	// the writer was not started with xmlTextWriterStartDocument
+	// the function adds always a newline
+	// if you load only a CDATA field then a newline is additional data
+	// additional data invalidates size information of the remote peer
+	// additional data crashs by this way all length checks if smlItemCheck !!!
 	
 	xmlFreeTextWriter(writer);
-	*string = (char *)xmlBufferContent(buffer);
+	*string = g_strndup(xmlBufferContent(buffer), xmlBufferLength(buffer));
 	if (size)
-		*size = xmlBufferLength(buffer) - 1;
+		*size = xmlBufferLength(buffer);
 	
-	//Hack to avoid memory copy
-	g_free(buffer);
+	xmlBufferFree(buffer);
 	
 	return TRUE;
 
@@ -247,7 +271,7 @@ error_free_buffer:
 	xmlBufferFree(buffer);
 error:
 	*string = NULL;
-	*size = 0;
+	if (size) *size = 0;
 	return FALSE;
 }
 
@@ -444,6 +468,55 @@ error:
 	return FALSE;
 }
 
+static SmlBool _smlChalMetaParse(SmlXmlParser *parser, char **format, char **type, char **nonce, SmlError **error)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p, %p, %p, %p)", __func__, parser, format, type, nonce, error);
+	smlAssert(parser);
+	
+	while (1) {
+		if (!_smlXmlParserStep(parser)) {
+			smlErrorSet(error, SML_ERROR_GENERIC, "Missing nodes");
+			goto error;
+		}
+
+		if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_META) && \
+		xmlTextReaderNodeType(parser->reader) == XML_NODE_CLOSE) {
+			break;
+		} else if (xmlTextReaderNodeType(parser->reader) != XML_NODE_START) {
+			smlErrorSet(error, SML_ERROR_GENERIC, "Not a start node");
+			goto error;
+		}
+		
+		if (type && !strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_TYPE)) {
+			if (!_smlXmlParserGetString(parser, type, SML_ELEMENT_TYPE, error))
+				goto error;
+		} else if (format && !strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_FORMAT)) {
+			if (!_smlXmlParserGetString(parser, format, SML_ELEMENT_FORMAT, error))
+				goto error;
+		} else if (format && !strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_NEXTNONCE)) {
+			if (!_smlXmlParserGetString(parser, nonce, SML_ELEMENT_NEXTNONCE, error))
+				goto error;
+		} else {
+			smlErrorSet(error, SML_ERROR_GENERIC, "wrong initial node: %s", xmlTextReaderConstName(parser->reader));
+			goto error;
+		}
+	}
+	
+	smlTrace(TRACE_EXIT, "%s", __func__);
+	return TRUE;
+
+error:
+	if (format)
+		*format = NULL;
+	if (nonce)
+		*nonce = NULL;
+	if (type)
+		*type = NULL;
+
+	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
+	return FALSE;
+}
+
 static SmlBool _smlCommandMetaParse(SmlXmlParser *parser, char **format, char **type, SmlAnchor **anchor, unsigned int *size, int *maxobjsize, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p, %p, %p, %p)", __func__, parser, format, type, anchor, size, maxobjsize, error);
@@ -573,6 +646,7 @@ static SmlItem *_smlItemParse(SmlXmlParser *parser, SmlCommand *cmd, SmlCommandT
 						goto error_free_item;
 					break;
 				case SML_COMMAND_TYPE_ADD:
+				case SML_COMMAND_TYPE_DELETE:	
 				case SML_COMMAND_TYPE_REPLACE:
 					if (!_smlCommandMetaParse(parser, NULL, &(item->contenttype), NULL, &cmd->size, NULL, error))
 						goto error_free_item;
@@ -671,15 +745,12 @@ static SmlCred *_smlCredParse(SmlXmlParser *parser, SmlError **error)
 	smlAssert(parser);
 	char *format = NULL;
 	char *type = NULL;
-	
-	SmlCred *cred = smlTryMalloc0(sizeof(SmlCred), error);
-	if (!cred)
-		goto error;
+	char *data = NULL;
 	
 	while (1) {
 		if (!_smlXmlParserStep(parser)) {
 			smlErrorSet(error, SML_ERROR_GENERIC, "Missing nodes");
-			goto error_free_cred;
+			goto error;
 		}
 		
 		if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_CRED) && \
@@ -687,40 +758,87 @@ static SmlCred *_smlCredParse(SmlXmlParser *parser, SmlError **error)
 			break;
 		} else if (xmlTextReaderNodeType(parser->reader) != XML_NODE_START) {
 			smlErrorSet(error, SML_ERROR_GENERIC, "Not a start node");
-			goto error_free_cred;
+			goto error;
 		}
 		
 		if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_META)) {
 			if (!_smlCommandMetaParse(parser, &format, &type, NULL, NULL, NULL, error))
-				goto error_free_cred;
+				goto error;
 		} else if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_DATA)) {
-			if (!_smlXmlParserGetString(parser, &cred->data, SML_ELEMENT_DATA, error))
-				goto error_free_cred;
+			if (!_smlXmlParserGetString(parser, &data, SML_ELEMENT_DATA, error))
+				goto error;
 		} else {
 			smlErrorSet(error, SML_ERROR_GENERIC, "wrong initial node %s", xmlTextReaderConstName(parser->reader));
-			goto error_free_cred;
+			goto error;
+		}
+	}
+
+	SmlCred *cred = smlCredNewFromString(type, format, data, error);
+	smlTrace(TRACE_EXIT, "%s: %p", __func__, cred);
+cleanup:
+	if (data)
+		g_free(data);
+	if (format)
+		g_free(format);
+	if (type)
+		g_free(type);
+	return cred;
+error:		
+	if (cred)
+		smlCredUnref(cred);
+	cred = NULL;
+	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
+	goto cleanup;
+}
+
+static SmlChal *_smlChalParse(SmlXmlParser *parser, SmlError **error)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %p)", __func__, parser, error);
+	smlAssert(parser);
+	char *format = NULL;
+	char *type = NULL;
+	
+	SmlChal *chal = smlTryMalloc0(sizeof(SmlChal), error);
+	if (!chal)
+		goto error;
+	
+	while (1) {
+		if (!_smlXmlParserStep(parser)) {
+			smlErrorSet(error, SML_ERROR_GENERIC, "Missing nodes");
+			goto error_free_chal;
+		}
+		
+		if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_CHAL) && \
+		xmlTextReaderNodeType(parser->reader) == XML_NODE_CLOSE) {
+			break;
+		} else if (xmlTextReaderNodeType(parser->reader) != XML_NODE_START) {
+			smlErrorSet(error, SML_ERROR_GENERIC, "Not a start node");
+			goto error_free_chal;
+		}
+		
+		if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_META)) {
+			if (!_smlChalMetaParse(parser, &format, &type, &chal->nonce, error))
+				goto error_free_chal;
+		} else {
+			smlErrorSet(error, SML_ERROR_GENERIC, "wrong initial node %s", xmlTextReaderConstName(parser->reader));
+			goto error_free_chal;
 		}
 	}
 		
-	if (!cred->data)  {
-		smlErrorSet(error, SML_ERROR_GENERIC, "Creds misformed");
-		goto error_free_cred;
-	}
-	
 	if (!format || !strcmp(format, SML_BASE64)) {
-		cred->format = SML_FORMAT_TYPE_BASE64;
+		chal->format = SML_FORMAT_TYPE_BASE64;
 	} else {
 		smlErrorSet(error, SML_ERROR_GENERIC, "Unknown format");
-		goto error_free_cred;
+		goto error_free_chal;
 	}
 	
 	if (!type || !strcmp(type, SML_AUTH_BASIC)) {
-		cred->type = SML_AUTH_TYPE_BASIC;
+		chal->type = SML_AUTH_TYPE_BASIC;
 	} else if (!strcmp(type, SML_AUTH_MD5)) {
-		cred->type = SML_AUTH_TYPE_MD5;
+		chal->type = SML_AUTH_TYPE_MD5;
 	} else {
 		smlErrorSet(error, SML_ERROR_GENERIC, "Unknown type");
-		goto error_free_cred;
+		goto error_free_chal;
 	}
 	
 	if (format)
@@ -728,11 +846,11 @@ static SmlCred *_smlCredParse(SmlXmlParser *parser, SmlError **error)
 	if (type)
 		g_free(type);
 	
-	smlTrace(TRACE_EXIT, "%s: %p", __func__, cred);
-	return cred;
+	smlTrace(TRACE_EXIT, "%s: %p", __func__, chal);
+	return chal;
 
-error_free_cred:
-	smlCredFree(cred);
+error_free_chal:
+	smlChalFree(chal);
 error:
 	if (format)
 		g_free(format);
@@ -851,12 +969,14 @@ error:
 	return FALSE;
 }
 
-static SmlBool _smlMessageParseSynchdrMeta(SmlXmlParser *parser, unsigned int *maxmsgsize, unsigned int *maxobjsize, SmlError **error)
+static SmlBool _smlMessageParseSynchdrMeta(SmlXmlParser *parser, unsigned int *maxmsgsize, unsigned int *maxobjsize, char **emi, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, parser, maxmsgsize, maxobjsize, error);
 	smlAssert(parser);
 	smlAssert(maxmsgsize);
 	smlAssert(maxobjsize);
+	smlAssert(emi);
+
 	
 	while (1) {
 		if (!_smlXmlParserStep(parser)) {
@@ -878,8 +998,11 @@ static SmlBool _smlMessageParseSynchdrMeta(SmlXmlParser *parser, unsigned int *m
 		} else if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_MAXOBJSIZE)) {
 			if (!_smlXmlParserGetID(parser, maxobjsize, SML_ELEMENT_MAXOBJSIZE, error))
 				goto error;
+		} else if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_EMI)) {
+			if (!_smlXmlParserGetString(parser, emi, SML_ELEMENT_EMI, error))
+				goto error;
 		} else {
-			smlErrorSet(error, SML_ERROR_GENERIC, "wrong initial node. expected MaxMsgSize");
+			smlErrorSet(error, SML_ERROR_GENERIC, "wrong initial node. expected MaxMsgSize, MaxObjSize or EMI. Instead of that: %s", xmlTextReaderConstName(parser->reader));
 			goto error;
 		}
 	}
@@ -1460,7 +1583,7 @@ void smlXmlParserFree(SmlXmlParser *parser)
 	if (parser->reader)
 		xmlFreeTextReader(parser->reader);
 	
-	xmlCleanupParser();
+	//xmlCleanupParser();
 	
 	g_free(parser);
 
@@ -1520,7 +1643,7 @@ SmlBool smlXmlParserGetHeader(SmlXmlParser *parser, SmlHeader **header, SmlCred
 			if (!_smlLocationParse(&((*header)->source), parser, error))
 				goto error_free_header;
 		} else if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_META)) {
-			if (!_smlMessageParseSynchdrMeta(parser, &((*header)->maxmsgsize), &((*header)->maxobjsize), error))
+			if (!_smlMessageParseSynchdrMeta(parser, &((*header)->maxmsgsize), &((*header)->maxobjsize), &((*header)->emi), error))
 				goto error_free_header;
 		} else if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_CRED)) {
 			*cred = _smlCredParse(parser, error);
@@ -1582,7 +1705,7 @@ error_free_header:
 	*header = NULL;
 	
 	if (*cred)
-		smlCredFree(*cred);
+		smlCredUnref(*cred);
 error:
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return FALSE;
@@ -1596,6 +1719,7 @@ SmlBool smlXmlParserGetStatus(SmlXmlParser *parser, SmlStatus **status, SmlError
 	
 	char *locURI = NULL;
 	SmlItem *item = NULL;
+	SmlChal *chal = NULL;
 	
 	/* Lets check if the next node is a command node */
 	if (smlCommandTypeFromString((char *)xmlTextReaderConstName(parser->reader), NULL) != SML_COMMAND_TYPE_UNKNOWN && xmlTextReaderNodeType(parser->reader) == XML_NODE_START) {
@@ -1648,6 +1772,14 @@ SmlBool smlXmlParserGetStatus(SmlXmlParser *parser, SmlStatus **status, SmlError
 			if (!_smlXmlParserExpectNode(parser, XML_NODE_CLOSE, FALSE, SML_ELEMENT_CMDID, error))
 				goto error_free_status;
 		} else if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_MSGREF)) {
+			/* MsgRef is defined as parsed character data:
+			   OMA-SUP-DTD_SyncML_RepPro-V1_2-20070221-A.txt:<!ELEMENT MsgRef (#PCDATA)>
+
+			   SyncML Reprent Version 1.1 20020215 - Chapter 5.1.13 MsgRef:
+			   "[...] reference to a SyncML session-uniquie identifier referenced by SyncML result or response status."
+
+			   So it could be something else then an unsigned integer.
+			*/
 			if (!_smlXmlParserGetID(parser, &((*status)->msgRef), SML_ELEMENT_MSGREF, error))
 				goto error_free_status;
 		} else if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_CMDREF)) {
@@ -1686,6 +1818,10 @@ SmlBool smlXmlParserGetStatus(SmlXmlParser *parser, SmlStatus **status, SmlError
 			item = _smlItemParse(parser, NULL, SML_COMMAND_TYPE_ALERT, error);
 			if (!item)
 				goto error_free_status;
+		} else if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_CHAL)) {
+			chal = _smlChalParse(parser, error);
+			if (!chal)
+				goto error_free_status;
 		} else {
 			smlErrorSet(error, SML_ERROR_GENERIC, "wrong initial node");
 			goto error;
@@ -1710,10 +1846,16 @@ SmlBool smlXmlParserGetStatus(SmlXmlParser *parser, SmlStatus **status, SmlError
 		smlItemUnref(item);
 	}
 	
+	/* MsgRef is defined as #PCDATA - so it doesn't have to be a
+	   unsigned integer AND it could be set to 0/zero as well.
+
+	   This sanity check breaks if MsgRef is set to 0/zero.
+
 	if (!(*status)->msgRef) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "No msgref set");
 		goto error_free_status;
 	}
+	*/
 	
 	if ((*status)->type == SML_COMMAND_TYPE_UNKNOWN) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "No cmd set");
@@ -1956,6 +2098,473 @@ error:
 	return FALSE;
 }
 
+static SmlBool _smlXmlDevInfDataStoreParseCTCap11(SmlXmlParser *parser, SmlDevInf *devinf, SmlError **error)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, parser, devinf, error);
+	smlAssert(parser);
+	smlAssert(devinf);
+
+	/* This function was designed to parse one CTCap section */
+	/* which can include several CTCap definitions           */
+
+	if (!_smlXmlParserStep(parser)) {
+		smlErrorSet(error, SML_ERROR_GENERIC, "Missing nodes");
+		goto error;
+	}
+
+	SmlDevInfCTCap *ctcap = NULL;
+	SmlDevInfProperty *property = NULL;
+	SmlDevInfPropParam *param = NULL;
+	char *value = NULL;
+	while (1) {
+
+		const char *elem_name;
+		elem_name = (const char*)xmlTextReaderConstName(parser->reader);
+
+		smlTrace(TRACE_INTERNAL, "read: %s\n", elem_name);
+		if (!strcmp(elem_name, SML_ELEMENT_CTCAP) &&
+			xmlTextReaderNodeType(parser->reader) == XML_NODE_CLOSE) {
+			// this is the end of the CTCap section
+			break;
+		}
+
+		if (xmlTextReaderNodeType(parser->reader) != XML_NODE_START)
+		{
+			smlErrorSet(error, SML_ERROR_GENERIC, "Not a start node: %s",
+				(char *)xmlTextReaderConstName(parser->reader));
+			goto error;
+		}
+
+		/* determine list item type */
+		SmlDevInfCTCapType type;
+		type = smlDevInfCTCapTypeFromString(elem_name, NULL);
+		value = NULL;
+		if (type != SML_DEVINF_CTCAP_UNKNOWN)
+		{
+			_smlXmlParserGetString(parser, &value, elem_name, error);
+		}
+
+		/* now react on the different items */
+		switch(type)
+		{
+			case SML_DEVINF_CTCAP_CTTYPE:
+				// CTType => new ctcap definition
+				if (ctcap != NULL)
+				{
+					if (property != NULL)
+                                	{
+                                        	if (param != NULL)
+                                        	{
+                                                	smlDevInfPropertyAddPropParam(property, param);
+                                                	param = NULL;
+                                        	}
+                                        	smlDevInfCTCapAddProperty(ctcap, property);
+						property = NULL;
+					}
+					smlDevInfAppendCTCap(devinf, ctcap);
+					ctcap = NULL;
+				}
+				ctcap = smlDevInfNewCTCap(error);
+				if (!ctcap)
+					goto error;
+				smlDevInfCTCapSetCTType(ctcap, value);
+				break;
+			case SML_DEVINF_CTCAP_VERCT:
+				if (ctcap == NULL)
+					goto error;
+				smlDevInfCTCapSetVerCT(ctcap, value);
+				break;
+			case SML_DEVINF_CTCAP_PROPNAME:
+				// PropName => new property
+				if (ctcap == NULL)
+					goto error;
+				if (property != NULL)
+                               	{
+                                       	if (param != NULL)
+                                       	{
+                                               	smlDevInfPropertyAddPropParam(property, param);
+                                               	param = NULL;
+                                       	}
+                                       	smlDevInfCTCapAddProperty(ctcap, property);
+					property = NULL;
+				}
+				property = smlDevInfNewProperty(error);
+				if (!property)
+					goto error;
+				smlDevInfPropertySetPropName(property, value);
+				break;
+			case SML_DEVINF_CTCAP_DATATYPE:
+				if (property == NULL)
+					goto error;
+				if (param != NULL)
+					smlDevInfPropParamSetDataType(param, value);
+				else
+					smlDevInfPropertySetDataType(property, value);
+				break;
+			case SML_DEVINF_CTCAP_MAXOCCUR:
+				if (property == NULL)
+					goto error;
+				smlDevInfPropertySetMaxOccur(property, atoi(value));
+				break;
+			case SML_DEVINF_CTCAP_MAXSIZE:
+				if (property == NULL)
+					goto error;
+				smlDevInfPropertySetMaxSize(property, atoi(value));
+				break;
+			case SML_DEVINF_CTCAP_NOTRUNCATE:
+				if (property == NULL)
+					goto error;
+				smlDevInfPropertySetNoTruncate(property);
+				break;
+			case SML_DEVINF_CTCAP_DISPLAYNAME:
+				if (property == NULL)
+					goto error;
+				if (param != NULL)
+					smlDevInfPropParamSetDisplayName(param, value);
+				else
+					smlDevInfPropertySetDisplayName(property, value);
+				break;
+			case SML_DEVINF_CTCAP_VALENUM:
+				if (property == NULL)
+					goto error;
+				if (param != NULL)
+					smlDevInfPropParamAddValEnum(param, value);
+				else
+					smlDevInfPropertyAddValEnum(property, value);
+				break;
+			case SML_DEVINF_CTCAP_PARAMNAME:
+				if (property == NULL)
+					goto error;
+                                if (param != NULL)
+                                {
+                                       	smlDevInfPropertyAddPropParam(property, param);
+                                       	param = NULL;
+                                }
+				param = smlDevInfNewPropParam(error);
+				if (!param)
+					goto error;
+				smlDevInfPropParamSetParamName(param, value);
+				break;
+			case SML_DEVINF_CTCAP_SIZE:
+				smlDevInfPropertySetPropSize(property, atoi(value));
+				break;
+			default:
+				smlErrorSet(error, SML_ERROR_GENERIC, "Unknown CTCapType: %s", elem_name);
+				goto error;
+		}
+		g_free(value);
+
+		if (!_smlXmlParserStep(parser)) {
+			smlErrorSet(error, SML_ERROR_GENERIC, "Missing nodes");
+			goto error;
+		}
+	}
+	if (param != NULL)
+		smlDevInfPropertyAddPropParam(property, param);
+	if (property != NULL)
+		smlDevInfCTCapAddProperty(ctcap, property);
+	if (ctcap != NULL)
+		smlDevInfAppendCTCap(devinf, ctcap);
+	
+	smlTrace(TRACE_EXIT, "%s", __func__);
+	return TRUE;
+
+error:
+	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
+	if (value != NULL)
+		g_free(value);
+	return FALSE;
+}
+
+static SmlDevInfPropParam *_smlXmlDevInfDataStoreParseCTCap12PropParam(SmlXmlParser *parser, SmlError **error)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, parser, error);
+	smlAssert(parser);
+
+	if (!_smlXmlParserStep(parser)) {
+		smlErrorSet(error, SML_ERROR_GENERIC, "Missing nodes");
+		goto error;
+	}
+
+	SmlDevInfPropParam *param = smlDevInfNewPropParam(error);
+	if (!param)
+		goto error;
+
+	char *value = NULL;
+	while (1) {
+
+		const char *elem_name;
+		elem_name = (const char*)xmlTextReaderConstName(parser->reader);
+
+		smlTrace(TRACE_INTERNAL, "read: %s\n", elem_name);
+		if (!strcmp(elem_name, SML_ELEMENT_PROPPARAM) &&
+			xmlTextReaderNodeType(parser->reader) == XML_NODE_CLOSE) {
+			// this is the end of the PropParam section
+			break;
+		}
+
+		if (xmlTextReaderNodeType(parser->reader) != XML_NODE_START)
+		{
+			smlErrorSet(error, SML_ERROR_GENERIC, "Not a start node: %s",
+				(char *)xmlTextReaderConstName(parser->reader));
+			goto error;
+		}
+
+		/* determine item type */
+		SmlDevInfCTCapType type;
+		type = smlDevInfCTCapTypeFromString(elem_name, NULL);
+		value = NULL;
+		if (type != SML_DEVINF_CTCAP_UNKNOWN)
+		{
+			_smlXmlParserGetString(parser, &value, elem_name, error);
+		}
+
+		/* now react on the different items */
+		switch(type)
+		{
+			case SML_DEVINF_CTCAP_PARAMNAME:
+				smlDevInfPropParamSetParamName(param, value);
+				break;
+			case SML_DEVINF_CTCAP_DATATYPE:
+				smlDevInfPropParamSetDataType(param, value);
+				break;
+			case SML_DEVINF_CTCAP_DISPLAYNAME:
+				smlDevInfPropParamSetDisplayName(param, value);
+				break;
+			case SML_DEVINF_CTCAP_VALENUM:
+				smlDevInfPropParamAddValEnum(param, value);
+				break;
+			default:
+				smlErrorSet(error, SML_ERROR_GENERIC, "Unknown CTCapType for PropParam: %s", elem_name);
+				goto error;
+		}
+		g_free(value);
+
+		if (!_smlXmlParserStep(parser)) {
+			smlErrorSet(error, SML_ERROR_GENERIC, "Missing nodes");
+			goto error;
+		}
+	}
+	
+	smlTrace(TRACE_EXIT, "%s", __func__);
+	return param;
+
+error:
+	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
+	if (value != NULL)
+		g_free(value);
+	return NULL;
+}
+
+static SmlDevInfProperty *_smlXmlDevInfDataStoreParseCTCap12Property(SmlXmlParser *parser, SmlError **error)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, parser, error);
+	smlAssert(parser);
+
+	if (!_smlXmlParserStep(parser)) {
+		smlErrorSet(error, SML_ERROR_GENERIC, "Missing nodes");
+		goto error;
+	}
+
+	SmlDevInfProperty *property = smlDevInfNewProperty(error);
+	if (!property)
+		goto error;
+
+	SmlDevInfPropParam *param;
+	char *value = NULL;
+	while (1) {
+
+		const char *elem_name;
+		elem_name = (const char*)xmlTextReaderConstName(parser->reader);
+
+		smlTrace(TRACE_INTERNAL, "read: %s\n", elem_name);
+		if (!strcmp(elem_name, SML_ELEMENT_PROPERTY) &&
+			xmlTextReaderNodeType(parser->reader) == XML_NODE_CLOSE) {
+			// this is the end of the Property section
+			break;
+		}
+
+		if (xmlTextReaderNodeType(parser->reader) != XML_NODE_START)
+		{
+			smlErrorSet(error, SML_ERROR_GENERIC, "Not a start node: %s",
+				(char *)xmlTextReaderConstName(parser->reader));
+			goto error;
+		}
+
+		/* determine item type */
+		SmlDevInfCTCapType type;
+		type = smlDevInfCTCapTypeFromString(elem_name, NULL);
+		value = NULL;
+		if (type != SML_DEVINF_CTCAP_UNKNOWN &&
+		    type != SML_DEVINF_CTCAP_PROPPARAM)
+		{
+			_smlXmlParserGetString(parser, &value, elem_name, error);
+		}
+
+		/* now react on the different items */
+		switch(type)
+		{
+			case SML_DEVINF_CTCAP_PROPNAME:
+				smlDevInfPropertySetPropName(property, value);
+				break;
+			case SML_DEVINF_CTCAP_DATATYPE:
+				smlDevInfPropertySetDataType(property, value);
+				break;
+			case SML_DEVINF_CTCAP_MAXOCCUR:
+				smlDevInfPropertySetMaxOccur(property, atoi(value));
+				break;
+			case SML_DEVINF_CTCAP_MAXSIZE:
+				smlDevInfPropertySetMaxSize(property, atoi(value));
+				break;
+			case SML_DEVINF_CTCAP_NOTRUNCATE:
+				smlDevInfPropertySetNoTruncate(property);
+				break;
+			case SML_DEVINF_CTCAP_DISPLAYNAME:
+				smlDevInfPropertySetDisplayName(property, value);
+				break;
+			case SML_DEVINF_CTCAP_VALENUM:
+				smlDevInfPropertyAddValEnum(property, value);
+				break;
+			case SML_DEVINF_CTCAP_PROPPARAM:
+				param = _smlXmlDevInfDataStoreParseCTCap12PropParam(parser, error);
+				if (!param)
+					goto error;
+				smlDevInfPropertyAddPropParam(property, param);
+				break;
+			case SML_DEVINF_CTCAP_SIZE:
+				smlDevInfPropertySetPropSize(property, atoi(value));
+				break;
+			default:
+				smlErrorSet(error, SML_ERROR_GENERIC, "Unknown CTCapType for Property: %s", elem_name);
+				goto error;
+		}
+		g_free(value);
+
+		if (!_smlXmlParserStep(parser)) {
+			smlErrorSet(error, SML_ERROR_GENERIC, "Missing nodes");
+			goto error;
+		}
+	}
+	
+	smlTrace(TRACE_EXIT, "%s", __func__);
+	return property;
+
+error:
+	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
+	if (value != NULL)
+		g_free(value);
+	return NULL;
+}
+
+static SmlBool _smlXmlDevInfDataStoreParseCTCap12(SmlXmlParser *parser, SmlDevInf *devinf, SmlError **error)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, parser, devinf, error);
+	smlAssert(parser);
+	smlAssert(devinf);
+
+	/* This function was designed to parse one CTCap section */
+	/* which can include only one CTCap definition           */
+
+	if (!_smlXmlParserStep(parser)) {
+		smlErrorSet(error, SML_ERROR_GENERIC, "Missing nodes");
+		goto error;
+	}
+
+	SmlDevInfCTCap *ctcap = smlDevInfNewCTCap(error);
+	if (!ctcap)
+		goto error;
+
+	SmlDevInfProperty *property;
+	char *value = NULL;
+	while (1) {
+
+		const char *elem_name;
+		elem_name = (const char*)xmlTextReaderConstName(parser->reader);
+
+		smlTrace(TRACE_INTERNAL, "read: %s\n", elem_name);
+		if (!strcmp(elem_name, SML_ELEMENT_CTCAP) &&
+			xmlTextReaderNodeType(parser->reader) == XML_NODE_CLOSE) {
+			// this is the end of the CTCap section
+			break;
+		}
+
+		if (xmlTextReaderNodeType(parser->reader) != XML_NODE_START)
+		{
+			smlErrorSet(error, SML_ERROR_GENERIC, "Not a start node: %s",
+				(char *)xmlTextReaderConstName(parser->reader));
+			goto error;
+		}
+
+		/* determine item type */
+		SmlDevInfCTCapType type;
+		type = smlDevInfCTCapTypeFromString(elem_name, NULL);
+		value = NULL;
+		if (type != SML_DEVINF_CTCAP_UNKNOWN &&
+		    type != SML_DEVINF_CTCAP_PROPERTY)
+		{
+			_smlXmlParserGetString(parser, &value, elem_name, error);
+		}
+
+		/* now react on the different items */
+		switch(type)
+		{
+			case SML_DEVINF_CTCAP_CTTYPE:
+				smlDevInfCTCapSetCTType(ctcap, value);
+				break;
+			case SML_DEVINF_CTCAP_VERCT:
+				smlDevInfCTCapSetVerCT(ctcap, value);
+				break;
+			case SML_DEVINF_CTCAP_PROPERTY:
+				property = _smlXmlDevInfDataStoreParseCTCap12Property(parser, error);
+				if (!property)
+					goto error;
+				smlDevInfCTCapAddProperty(ctcap, property);
+				break;
+			default:
+				smlErrorSet(error, SML_ERROR_GENERIC, "Unknown CTCapType for CTCap: %s", elem_name);
+				goto error;
+		}
+		g_free(value);
+
+		if (!_smlXmlParserStep(parser)) {
+			smlErrorSet(error, SML_ERROR_GENERIC, "Missing nodes");
+			goto error;
+		}
+	}
+	smlDevInfAppendCTCap(devinf, ctcap);
+	
+	smlTrace(TRACE_EXIT, "%s", __func__);
+	return TRUE;
+
+error:
+	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
+	if (value != NULL)
+		g_free(value);
+	return FALSE;
+}
+
+static SmlBool _smlXmlDevInfDataStoreParseCTCap(SmlXmlParser *parser, SmlDevInf *devinf, SmlError **error)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, parser, devinf, error);
+	smlAssert(parser);
+	smlAssert(devinf);
+
+	/* SyncML 1.0 and 1.1 start a new CTCap with a new CTType definition */
+	/* SyncML 1.2 starts a new CTCap with a new CTCap definition */
+	/* SyncML 1.2 attaches the CTCaps to the appropriate datastore */
+	/* SyncML 1.0 and 1.1 attach the CTCaps to DevInf */
+	/* The SyncML version is in devinf->version */
+
+	SmlBool ret;
+	if (devinf->version == SML_DEVINF_VERSION_12)
+		ret = _smlXmlDevInfDataStoreParseCTCap12(parser, devinf, error);
+	else
+		ret = _smlXmlDevInfDataStoreParseCTCap11(parser, devinf, error);
+
+	smlTrace(TRACE_EXIT, "%s (%d)", __func__, ret);
+	return ret;
+}
+
 static SmlBool _smlXmlDevInfDataStoreParseRxTx(SmlXmlParser *parser, const char *element, char **cttype, char **version, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %s, %p, %p, %p)", __func__, parser, element, cttype, version, error);
@@ -2029,6 +2638,8 @@ static SmlBool _smlXmlDevInfDataStoreParse(SmlXmlParser *parser, SmlDevInf *devi
 	if (!datastore)
 		goto error;
 	datastore->refCount = 1;
+	datastore->rxPrefContentType = NULL;
+	datastore->txPrefContentType = NULL;
 	
 	if (!_smlXmlParserStep(parser)) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "Missing nodes");
@@ -2066,6 +2677,9 @@ static SmlBool _smlXmlDevInfDataStoreParse(SmlXmlParser *parser, SmlDevInf *devi
 		} else if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_TX)) {
 			if (!_smlXmlDevInfDataStoreParseRxTx(parser, SML_ELEMENT_TX, &(datastore->txContentType), &(datastore->txVersion), error))
 				goto error_free_datastore;
+		} else if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_CTCAP)) {
+			if (!_smlXmlDevInfDataStoreParseCTCap(parser, devinf, error))
+				goto error_free_datastore;
 		} else if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_SYNCCAP)) {
 			if (!_smlXmlDevInfDataStoreParseSyncCap(parser, datastore, error))
 				goto error_free_datastore;
@@ -2114,7 +2728,7 @@ SmlDevInf *smlXmlDevInfParse(const char *data, unsigned int size, SmlError **err
 	parser->reader = xmlReaderForMemory(data, size, "/", NULL, XML_PARSE_NONET | XML_PARSE_NOERROR | XML_PARSE_NOWARNING | XML_PARSE_NOCDATA | XML_PARSER_SUBST_ENTITIES);
 	if (!parser->reader) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "Unable to create new reader");
-		goto error;
+		goto error_free_parser;
 	}
 	xmlSubstituteEntitiesDefault(1);
 	
@@ -2125,6 +2739,8 @@ SmlDevInf *smlXmlDevInfParse(const char *data, unsigned int size, SmlError **err
 	SmlDevInf *devinf = smlTryMalloc0(sizeof(SmlDevInf), error);
 	if (!devinf)
 		goto error_free_reader;
+
+	smlDevInfRef(devinf);
 	
 	if (!_smlXmlParserStep(parser)) {
 		smlErrorSet(error, SML_ERROR_GENERIC, "Missing nodes");
@@ -2150,6 +2766,8 @@ SmlDevInf *smlXmlDevInfParse(const char *data, unsigned int size, SmlError **err
 				devinf->version = SML_DEVINF_VERSION_11;
 			else if (verdtd && !strcmp(verdtd, "1.0"))
 				devinf->version = SML_DEVINF_VERSION_10;
+			else if (verdtd && !strcmp(verdtd, "1.2"))
+				devinf->version = SML_DEVINF_VERSION_12;
 			else {
 				g_free(verdtd);
 				smlErrorSet(error, SML_ERROR_GENERIC, "Unknown devinf version");
@@ -2251,19 +2869,14 @@ SmlDevInf *smlXmlDevInfParse(const char *data, unsigned int size, SmlError **err
 			if (!_smlXmlDevInfDataStoreParse(parser, devinf, error))
 				goto error_free_devinf;
 		} else if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_CTCAP)) {
-			/* Ignored for now */
-			smlTrace(TRACE_INTERNAL, "Skipping ctcap node");
-			if (xmlTextReaderNext(parser->reader) != 1) {
-				smlErrorSet(error, SML_ERROR_GENERIC, "Unable to skip ctcap node");
-				goto error;
-			}
-			continue;
+			if (!_smlXmlDevInfDataStoreParseCTCap(parser, devinf, error))
+				goto error_free_devinf;
 		} else if (!strcmp((char *)xmlTextReaderConstName(parser->reader), SML_ELEMENT_EXT)) {
 			/* Ignored for now */
 			smlTrace(TRACE_INTERNAL, "Skipping ext node");
 			if (xmlTextReaderNext(parser->reader) != 1) {
 				smlErrorSet(error, SML_ERROR_GENERIC, "Unable to skip ext node");
-				goto error;
+				goto error_free_parser;
 			}
 			continue;
 		} else {
@@ -2279,6 +2892,7 @@ SmlDevInf *smlXmlDevInfParse(const char *data, unsigned int size, SmlError **err
 		
 	xmlFreeTextReader(parser->reader);
 	parser->reader = NULL;
+	g_free(parser);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return devinf;
@@ -2287,8 +2901,10 @@ error_free_devinf:
 	smlDevInfUnref(devinf);
 error_free_reader:
 	xmlFreeTextReader(parser->reader);
-error:
+error_free_parser:
 	parser->reader = NULL;
+	g_free(parser);
+error:
 	smlTrace(TRACE_EXIT_ERROR, "%s: %s", __func__, smlErrorPrint(error));
 	return FALSE;
 }
