@@ -82,12 +82,15 @@ typedef enum {
 	SML_ERROR_USE_PROXY = 305,
 	
 	/* Errors */
-	SML_ERROR_BAD_REQUEST = 400,
-	SML_ERROR_AUTH_REJECTED = 401,
-	SML_ERROR_FORBIDDEN = 403,
-	SML_ERROR_NOT_FOUND = 404,
-	SML_ERROR_AUTH_REQUIRED = 407,
-	SML_ERROR_SIZE_MISMATCH = 424,
+	SML_ERROR_BAD_REQUEST = 400,		// Bad Request
+	SML_ERROR_AUTH_REJECTED = 401,		// Unauthorized, Invalid Credentials
+	SML_ERROR_PAYMENT_NEEDED =402,		// Payment need
+	SML_ERROR_FORBIDDEN = 403,		// Forbidden
+	SML_ERROR_NOT_FOUND = 404,		// Not found
+	SML_ERROR_COMMAND_NOT_ALLOWED = 405,	// Command not allowed
+	SML_ERROR_UNSUPPORTED_FEATURE = 406,	// Optional feature unsupported
+	SML_ERROR_AUTH_REQUIRED = 407,		// Authentication required, Missing Credentials
+	SML_ERROR_SIZE_MISMATCH = 424,		// Size mismatch
 	
 	/* Standard errors */
 	SML_ERROR_GENERIC = 500,
