@@ -33,6 +33,14 @@ typedef struct managerSession {
 	GList *objects;
 } managerSession;
 
+SmlObject *smlManagerObjectFindInternal(
+		SmlManager *manager,
+		SmlSession *session,
+		SmlCommandType type,
+		SmlLocation *target,
+		SmlLocation *source,
+		const char* contentType);
+
 static managerSession *_manager_session_find(SmlManager *manager, SmlSession *session)
 {
 	smlAssert(session);
@@ -111,16 +119,22 @@ static void _smlManagerSendEvent(SmlManager *manager, SmlManagerEventType type,
 		if (session) {
 			event->session = session;
 			smlSessionRef(session);
+		} else {
+			event->session = NULL;
 		}
 		
 		if (command) {
 			event->command = command;
 			smlCommandRef(command);
+		} else {
+			event->command = NULL;
 		}
 		
 		if (parent) {
 			event->parent = parent;
 			smlCommandRef(parent);
+		} else {
+			event->parent = NULL;
 		}
 		
 		if (error) {
@@ -197,7 +211,9 @@ static SmlBool _smlManagerDataHandler(SmlTransport *tsp, SmlLink *link, SmlTrans
 			/* Find the session if available otherwise register it as new */
 			session = smlManagerSessionFind(manager, header->sessionID);
 			if (!session) {
-				if (!(session = smlSessionNew(SML_SESSION_TYPE_SERVER, data->type, header->version, header->protocol, header->target, header->source, header->sessionID, header->messageID + 1, &locerror)))
+				/* If we use the target and source from the header of the received message
+				 * then we MUST interchange source and target */
+				if (!(session = smlSessionNew(SML_SESSION_TYPE_SERVER, data->type, header->version, header->protocol, header->source, header->target, header->sessionID, header->messageID + 1, &locerror)))
 					goto error_free_header;
 				
 				if (!smlManagerSessionAdd(manager, session, link, &locerror))
@@ -232,7 +248,7 @@ static SmlBool _smlManagerDataHandler(SmlTransport *tsp, SmlLink *link, SmlTrans
 			
 			smlHeaderFree(header);
 			if (cred)
-				smlCredFree(cred);
+				smlCredUnref(cred);
 			
 			/* Now let the session handle the commands etc */
 			if (!smlSessionReceiveBody(session, parser, &locerror))
@@ -255,7 +271,7 @@ error_free_header:
 	if (header)
 		smlHeaderFree(header);
 	if (cred)
-		smlCredFree(cred);
+		smlCredUnref(cred);
 error_free_parser:
 	smlParserFree(parser);
 session_error:
@@ -284,8 +300,15 @@ SmlManager *smlManagerNew(SmlTransport *tsp, SmlError **error)
 	manager->transport = tsp;
 	
 	manager->context = g_main_context_new();
-	tsp->context = manager->context;
-	g_main_context_ref(tsp->context);
+	if (tsp->context == NULL &&
+            // tsp->type != SML_TRANSPORT_HTTP_SERVER &&
+	    tsp->type != SML_TRANSPORT_HTTP_CLIENT)
+	{
+		// if the context is already set
+		// then the transport is already running in asynchronous mode
+		tsp->context = manager->context;
+		g_main_context_ref(tsp->context);
+	}
 	
 	manager->functions = smlTryMalloc0(sizeof(GSourceFuncs), error);
 	if (!manager->functions)
@@ -335,6 +358,7 @@ void smlManagerFree(SmlManager *manager)
 			_smlManagerEventFree(event);
 		
 		smlQueueFree(manager->userEventQueue);
+		manager->userEventQueue = NULL;
 	}
 
 	if (manager->functions)
@@ -448,12 +472,15 @@ SmlSession *smlManagerSessionFind(SmlManager *manager, const char *sessionID)
 
 void smlManagerDispatch(SmlManager *manager)
 {
+	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, manager);
+	smlAssert(manager);
 	SmlManagerEvent *event = NULL;
 	while ((event = smlQueueTryPop(manager->userEventQueue))) {
 		smlAssert(manager->eventCallback);
 		manager->eventCallback(manager, event->type, event->session, event->error, manager->eventCallbackUserdata);
 		_smlManagerEventFree(event);
 	}
+	smlTrace(TRACE_EXIT, "%s", __func__);
 }
 
 SmlBool smlManagerCheck(SmlManager *manager)
@@ -470,14 +497,14 @@ static void _event_callback(SmlSession *session, SmlSessionEventType type, SmlCo
 	switch (type) {
 		case SML_SESSION_EVENT_HEADER_REPLY:
 			break;
-		case SML_SESSION_EVENT_CHILD_COMMAND:
-			if (!smlManagerDispatchChildCommand(manager, session, parent, command, &locerror)) {
+		case SML_SESSION_EVENT_COMMAND:
+			if (!smlManagerDispatchCommand(manager, session, command, &locerror)) {
 				_smlManagerSendEvent(manager, SML_MANAGER_SESSION_WARNING, session, NULL, NULL, locerror);
 				goto error;
 			}
 			break;
-		case SML_SESSION_EVENT_COMMAND:
-			if (!smlManagerDispatchCommand(manager, session, command, &locerror)) {
+		case SML_SESSION_EVENT_CHILD_COMMAND:
+			if (!smlManagerDispatchChildCommand(manager, session, parent, command, &locerror)) {
 				_smlManagerSendEvent(manager, SML_MANAGER_SESSION_WARNING, session, NULL, NULL, locerror);
 				goto error;
 			}
@@ -613,11 +640,62 @@ SmlBool smlManagerObjectRegister(SmlManager *manager, SmlCommandType type, SmlSe
 	smlTrace(TRACE_ENTRY, "%s(%p, %i, %p, %p, %p, %s, %p, %p, %p, %p)", __func__, manager, type, session, location, source, contentType, callback, childCallback, userdata, error);
 	smlAssert(manager);
 	smlAssert(callback);
+
+	/* first we have to check if this an update for an already registered object */
+	/* this is important for actions which require more than one DsSession */
 	
-	SmlObject *object = smlTryMalloc0(sizeof(SmlObject), error);
-	if (!object)
-		goto error;
-	
+        SmlObject *object = smlManagerObjectFindInternal(
+				manager, session, type,
+				location, source, contentType);
+	if (object != NULL && session != NULL)
+	{
+		/* check that it is not a global manager object */
+		SmlObject *hObject = smlManagerObjectFindInternal(
+                                	manager, NULL, type,
+                                	location, source, contentType);
+		if (hObject != NULL && hObject == object)
+		{
+			/* the found object is a global manager object */
+			/* so we have to register a new object */
+			object = NULL;
+		}
+	}
+	if (object)
+	{
+		/* there is an object and so we reconfigure it */
+		smlTrace(TRACE_INTERNAL, "prepare a reusable object");
+
+		if (object->location)
+			smlLocationUnref(object->location);
+		if (object->source)
+			smlLocationUnref(object->source);
+		if (object->contentType)
+			g_free(object->contentType);
+	}
+	else
+	{
+		/* let's create and register a new object */
+		smlTrace(TRACE_INTERNAL, "create and register a new object");
+
+		object = smlTryMalloc0(sizeof(SmlObject), error);
+		if (!object)
+			goto error;
+
+		if (session) {
+			managerSession *sess = _manager_session_find(manager, session);
+			if (!sess) {
+				smlErrorSet(error, SML_ERROR_GENERIC, "Session not found");
+				goto error_free_object;
+			}
+		
+			sess->objects = g_list_append(sess->objects, object);
+		} else {
+			manager->objects = g_list_append(manager->objects, object);
+		}
+	}
+
+	smlTrace(TRACE_INTERNAL, "configure registered object");
+
 	object->type = type;
 	
 	if (location) {
@@ -638,17 +716,6 @@ SmlBool smlManagerObjectRegister(SmlManager *manager, SmlCommandType type, SmlSe
 	object->childCallback = childCallback;
 	object->commandCallbackUserdata = userdata;
 	
-	if (session) {
-		managerSession *sess = _manager_session_find(manager, session);
-		if (!sess) {
-			smlErrorSet(error, SML_ERROR_GENERIC, "Session not found");
-			goto error_free_object;
-		}
-		
-		sess->objects = g_list_append(sess->objects, object);
-	} else
-		manager->objects = g_list_append(manager->objects, object);
-	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
@@ -710,8 +777,28 @@ SmlObject *smlManagerObjectFind(SmlManager *manager, SmlSession *session, SmlCom
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, manager, session, cmd);
 	smlAssert(manager);
-	smlAssert(session);
 	smlAssert(cmd);
+
+	SmlObject *object = smlManagerObjectFindInternal(
+				manager,
+				session,
+				cmd->type,
+				cmd->target, cmd->source,
+				cmd->private.alert.contentType);
+	smlTrace(TRACE_EXIT, "%s(%p)", __func__, object);
+	return object;
+}
+
+SmlObject *smlManagerObjectFindInternal(
+		SmlManager *manager,
+		SmlSession *session,
+		SmlCommandType type,
+		SmlLocation *target,
+		SmlLocation *source,
+		const char* contentType)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %d, %p, %p)", __func__, manager, session, type, target, source);
+	smlAssert(manager);
 	SmlObject *object = NULL;
 	GList *o = NULL;
 	
@@ -722,13 +809,13 @@ SmlObject *smlManagerObjectFind(SmlManager *manager, SmlSession *session, SmlCom
 			for (o = sess->objects; o; o = o->next) {
 				object = o->data;
 				
-				if (object->type != SML_COMMAND_TYPE_UNKNOWN && cmd->type != object->type)
+				if (object->type != SML_COMMAND_TYPE_UNKNOWN && type != object->type)
 					continue;
 				
-				if (!smlLocationCompare(NULL, object->location, NULL, cmd->target))
+				if (!smlLocationCompare(NULL, object->location, NULL, target))
 					continue;
 				
-				if (!smlLocationCompare(NULL, object->source, NULL, cmd->source))
+				if (!smlLocationCompare(NULL, object->source, NULL, source))
 					continue;
 				
 				smlTrace(TRACE_EXIT, "%s: FOUND (session): %p", __func__, object);
@@ -740,13 +827,13 @@ SmlObject *smlManagerObjectFind(SmlManager *manager, SmlSession *session, SmlCom
 	for (o = manager->objects; o; o = o->next) {
 		object = o->data;
 		
-		if (object->type != SML_COMMAND_TYPE_UNKNOWN && cmd->type != object->type)
+		if (object->type != SML_COMMAND_TYPE_UNKNOWN && type != object->type)
 			continue;
 		
-		if (cmd->type == SML_COMMAND_TYPE_ALERT && cmd->private.alert.contentType) {
+		if (type == SML_COMMAND_TYPE_ALERT && contentType) {
 			if (object->contentType) {
 				/* This item is a san 11 alert */
-				if (!strcmp(cmd->private.alert.contentType, object->contentType)) {
+				if (!strcmp(contentType, object->contentType)) {
 					smlTrace(TRACE_EXIT, "%s: FOUND SAN TARGET: %p", __func__, object);
 					return object;
 				}
@@ -755,10 +842,10 @@ SmlObject *smlManagerObjectFind(SmlManager *manager, SmlSession *session, SmlCom
 			continue;
 		}
 		
-		if (!smlLocationCompare(NULL, object->location, NULL, cmd->target))
+		if (!smlLocationCompare(NULL, object->location, NULL, target))
 			continue;
 		
-		if (!smlLocationCompare(NULL, object->source, NULL, cmd->source))
+		if (!smlLocationCompare(NULL, object->source, NULL, source))
 			continue;
 		
 		/* Check if the target is a san target */
@@ -876,7 +963,10 @@ SmlBool smlManagerDispatchCommand(SmlManager *manager, SmlSession *session, SmlC
 		
 		object->commandCallback(session, cmd, object->commandCallbackUserdata);
 	} else {
-		smlErrorSet(error, SML_ERROR_NOT_FOUND, "Unable to find command handler");
+		const char *type = smlCommandTypeToString(cmd->type, NULL);
+		const char *srcuri = (cmd->source && cmd->source->locURI) ? cmd->source->locURI : "NULL";
+		const char *dsturi = (cmd->target && cmd->target->locURI) ? cmd->target->locURI : "NULL";
+		smlErrorSet(error, SML_ERROR_NOT_FOUND, "Unable to find command handler (%s: %s -> %s)", (type) ? type : "UNKNOWN", srcuri, dsturi);
 		
 		SmlStatus *reply = smlCommandNewReply(cmd, SML_ERROR_NOT_FOUND, error);
 		if (!reply)
