@@ -30,6 +30,9 @@ struct SmlNotification {
 	SmlLocation *target;
 	GList *alerts;
 	SmlMimeType type;
+	SmlCred *cred;
+	SmlSessionType sessionType;
+	SmlManager *manager;
 };
 
 struct SmlSanAlert {
