@@ -756,8 +756,8 @@ SmlAssemblerResult smlAssemblerAddStatusFull(SmlAssembler *assm, SmlStatus *stat
 		}
 	}
 	
-	if (status->cmdRef != 0)
-		assm->empty = FALSE;
+	/*if (status->cmdRef != 0)
+		assm->empty = FALSE;*/
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return SML_ASSEMBLER_RESULT_OK;
@@ -836,6 +836,13 @@ void smlAssemblerSetRequestedLimit(SmlAssembler *assm, unsigned int limit)
 	smlTrace(TRACE_ENTRY, "%s(%p, %u)", __func__, assm, limit);
 	smlAssert(assm);
 	
+	/* Dont update the limit if it is 0. A client might send a maxmsg size
+	 * in the first package, and none in the second. Then, the maxmsgsize from
+	 * the first package has to be used. See Syncml 1.1 MetInf Section 5.8 */
+	if (limit == 0) {
+		smlTrace(TRACE_EXIT, "%s: Not setting limit. limit is 0", __func__);
+		return;
+	}
 	assm->req_limit = limit;
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
@@ -857,7 +864,7 @@ void smlAssemblerSetLimit(SmlAssembler *assm, unsigned int limit)
  * returns the lower one of the 2 */
 int smlAssemblerGetSendingMaxObjSize(SmlAssembler *assm)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %u)", __func__, assm);
+	smlTrace(TRACE_ENTRY, "%s(%p)", __func__, assm);
 	smlAssert(assm);
 	int ret = 0;
 	
