@@ -1,4 +1,4 @@
-                                 /*
+/*
  * libsyncml - A syncml protocol implementation
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
  *
@@ -106,7 +106,13 @@ static SmlBool _smlSessionFlushInternal(SmlSession *session, SmlBool final, SmlE
 		goto error;
 	
 	/* Flush and reset the command ID counter */
-	session->lastCommandID = smlAssemblerFlush(session->assembler) - 1;
+	if (session->lastCommandID > 1)
+	{
+		smlAssemblerFlush(session->assembler) - 1;
+	} else {
+		session->lastCommandID = smlAssemblerFlush(session->assembler) - 1;
+	}
+	smlTrace(TRACE_INTERNAL, "flushed assembler - last command id is %i now", session->lastCommandID);
 	
 	SmlTransportData *tspdata = smlTransportDataNew(data, size, session->type, TRUE, error);
 	if (!tspdata)
@@ -347,6 +353,7 @@ static SmlAssemblerResult _smlSessionStartCommandInternal(SmlSession *session, S
 	/* We now increment the session ID */
 	session->lastCommandID++;
 	cmd->cmdID = session->lastCommandID;
+	smlTrace(TRACE_INTERNAL, "%s: last command id is %i", __func__, session->lastCommandID);
 	
 	/* Now we can try to add the command to the assembler */
 	switch (smlAssemblerStartCommand(session->assembler, parent, cmd, error)) {
@@ -473,12 +480,14 @@ static void _smlSessionCommandFree(SmlSessionCommand *sesscmd)
 
 static void _smlSessionCommandHandler(void *message, void *userdata)
 {
+	smlTrace(TRACE_ENTRY, "%s", __func__);
 	smlAssert(message);
 	smlAssert(userdata);
 	SmlSessionCommand *sesscmd = message;
 	SmlSession *session = userdata;
 	SmlError *error = NULL;
 	//SmlPendingStatus *pending = NULL;
+	smlTrace(TRACE_INTERNAL, "%s: session cmd type %i", __func__, sesscmd->type);
 	
 	SmlCommand *parent = sesscmd->parent;
 	SmlCommand *cmd = sesscmd->cmd;
@@ -519,6 +528,7 @@ static void _smlSessionCommandHandler(void *message, void *userdata)
 			if (status->cmdRef != cmdRef || status->msgRef != msgRef) {
 				//Put it on the end of the queue
 				smlQueueSendPrio(session->command_queue, message);
+				smlTrace(TRACE_EXIT, "%s", __func__);
 				return;
 			}
 			
@@ -544,6 +554,7 @@ static void _smlSessionCommandHandler(void *message, void *userdata)
 					
 					/* Push the message back into the queue */
 					smlQueuePushHeadPrio(session->command_queue, message);
+					smlTrace(TRACE_EXIT, "%s", __func__);
 					return;
 					break;
 				case SML_ASSEMBLER_RESULT_ERROR:
@@ -553,6 +564,7 @@ static void _smlSessionCommandHandler(void *message, void *userdata)
 	}
 	
 	_smlSessionCommandFree(sesscmd);
+	smlTrace(TRACE_EXIT, "%s", __func__);
 	
 	return;
 
@@ -578,6 +590,10 @@ SmlBool smlSessionReceiveHeader(SmlSession *session, SmlHeader *header, SmlError
 	session->lastCommandID++;
 	
 	smlAssemblerSetRequestedLimit(session->assembler, header->maxmsgsize);
+	if (header->maxmsgsize > 0 && header->maxmsgsize < session->incomingMaxMsgSize)
+	{
+		smlSessionSetReceivingMaxMsgSize(session, header->maxmsgsize);
+	}
 	
 	/* Reserve the status for header reply. It will always be cmdRef 0
 	 * and it will always be the first command in the message */
@@ -598,7 +614,23 @@ SmlBool smlSessionDispatchStatus(SmlSession *session, SmlStatus *status, SmlErro
 	smlAssert(status);
 	
 	if (status->cmdRef == 0) {
-		smlSessionDispatchEvent(session, SML_SESSION_EVENT_HEADER_REPLY, NULL, NULL, status, NULL);
+		smlAssert(status->type == SML_COMMAND_TYPE_HEADER);
+		smlAssert(status->data);
+		SmlErrorType errorCode = atoi(status->data);
+		if (errorCode == SML_AUTH_ACCEPTED)
+		{
+			// authentication succeeded
+			// we can remove the credentials from this session
+			// we do it now for better security
+			if (session->cred != NULL)
+			{
+				smlCredUnref(session->cred);
+				session->cred = NULL;
+			}
+		} else {
+			SmlCommand *cmd = smlCommandNew(SML_COMMAND_TYPE_HEADER, error);
+			smlSessionDispatchEvent(session, SML_SESSION_EVENT_HEADER_REPLY, cmd, NULL, status, NULL);
+		}
 		goto out;
 	}
 	
@@ -641,17 +673,33 @@ SmlBool smlSessionCheck(SmlSession *session)
 	smlAssert(session);
 	
 	if (session->waiting)
+	{
+		smlTrace(TRACE_INTERNAL, "%s: waiting", __func__);
 		return FALSE;
-	
+	}
+
+	SmlBool result;
 	if (!smlAssemblerIsStatusMissing(session->assembler))
-		return smlQueueCheck(session->command_queue);
+	{
+		result = smlQueueCheck(session->command_queue);
+		smlTrace(TRACE_INTERNAL, "%s: no status is missing - %i", __func__, result);
+	}
 	else
-		return smlQueueCheckPrio(session->command_queue);
+	{
+		result = smlQueueCheckPrio(session->command_queue);
+		smlTrace(TRACE_INTERNAL, "%s: status is missing - %i", __func__, result);
+	}
+	return result;
 }
 
 void smlSessionDispatch(SmlSession *session)
 {
+	smlTrace(TRACE_ENTRY, "%s", __func__);
 	smlAssert(session);
+	// dispatching does not work in parallel
+	// if SmlManager and SmlSession->ReceiveBody call dispatch
+	// then this can lead to surprising effects
+	g_mutex_lock(session->dispatch_mutex);
 	
 	if (session->waiting)
 		return;
@@ -660,9 +708,17 @@ void smlSessionDispatch(SmlSession *session)
 	 * If we are complete on the status side, we dispatch the commands
 	 */
 	if (!smlAssemblerIsStatusMissing(session->assembler))
+	{
 		smlQueueDispatch(session->command_queue);
+		smlTrace(TRACE_INTERNAL, "%s: dispatch commands", __func__);
+	}
 	else
+	{
 		smlQueueDispatchPrio(session->command_queue);
+		smlTrace(TRACE_INTERNAL, "%s: dispatch status", __func__);
+	}
+	g_mutex_unlock(session->dispatch_mutex);
+	smlTrace(TRACE_EXIT, "%s", __func__);
 }
 
 SmlBool smlSessionReceiveBody(SmlSession *session, SmlParser *parser, SmlError **error)
@@ -716,7 +772,11 @@ parse_status:
 				 * large object handling and support of number of changes. This way we dont
 				 * need to parse the devinf */
 				if (cmd->type == SML_COMMAND_TYPE_SYNC) {
-					smlTrace(TRACE_INTERNAL, "maxObjSize: %i, numbOfChanges: %i, %i", cmd->private.sync.maxObjSize, cmd->private.sync.hasNumChanged, cmd->private.sync.numChanged);
+					smlTrace(TRACE_INTERNAL, "%s: maxObjSize: %i, numbOfChanges: %i, %i",
+						__func__,
+						cmd->private.sync.maxObjSize,
+						cmd->private.sync.hasNumChanged,
+						cmd->private.sync.numChanged);
 					/* If the sync has the maxObjSize set (>= 0), we set the requested max obj size.
 					 * If the max obj size was set before (with the alert, we overwrite it */
 					if (cmd->private.sync.maxObjSize >= 0)
@@ -735,7 +795,8 @@ parse_status:
 				if (!session->end) {
 					smlSessionDispatchEvent(session, SML_SESSION_EVENT_COMMAND, cmd, NULL, NULL, NULL);
 				} else {
-					smlTrace(TRACE_INTERNAL, "Replying with 407 since the session has ended");
+					smlTrace(TRACE_INTERNAL, "%s: Replying with 407 since the session has ended",
+						__func__);
 					SmlStatus *reply = smlCommandNewReply(cmd, SML_ERROR_AUTH_REQUIRED, error);
 					if (!reply)
 						goto error_free_cmd;
@@ -767,7 +828,8 @@ parse_status:
 						smlCommandUnref(cmd);
 						break;
 					} else {
-						smlTrace(TRACE_INTERNAL, "Alert maxObjSize: %i", cmd->private.alert.maxObjSize);
+						smlTrace(TRACE_INTERNAL, "%s: Alert maxObjSize: %i",
+							__func__, cmd->private.alert.maxObjSize);
 						if (cmd->private.alert.maxObjSize >= 0)
 							smlAssemblerSetRequestedMaxObjSize(session->assembler, cmd->private.alert.maxObjSize);
 					}
@@ -777,7 +839,7 @@ parse_status:
 				if (cmd->type == SML_COMMAND_TYPE_ADD || cmd->type == SML_COMMAND_TYPE_REPLACE) {
 					
 					if (session->incomingBuffer) {
-						smlTrace(TRACE_INTERNAL, "Appending to item buffer");
+						smlTrace(TRACE_INTERNAL, "%s: Appending to item buffer", __func__);
 						/* We have a command in the incoming buffer. So we append the data to it.
 						 * If the new command does not have moreData set, we dispatch the command */
 
@@ -795,10 +857,14 @@ parse_status:
 						if (!smlItemAddData(session->incomingBuffer, data, size, error))
 							goto error_free_cmd;
 						
-						smlTrace(TRACE_INTERNAL, "Appended %i to buffer. Buffer size: %i. Required: %i", size, xmlBufferLength(session->incomingBuffer->buffer), session->incomingBuffer->size);
+						smlTrace(TRACE_INTERNAL, "%s: Appended %i to buffer. Buffer size: %i. Required: %i",
+							__func__, size,
+							xmlBufferLength(session->incomingBuffer->buffer),
+							session->incomingBuffer->size);
 						
 						if (!cmd->private.change.item->moreData) {
-							smlTrace(TRACE_INTERNAL, "Item buffer complete. Dispatching.");
+							smlTrace(TRACE_INTERNAL, "%s: Item buffer complete. Dispatching.",
+								__func__);
 							/* This cmd doesnt have more data set. So we check if the data
 							 * is complete and dispatch it if it is */
 							if (!smlItemCheck(session->incomingBuffer)) {
@@ -833,15 +899,20 @@ parse_status:
 						
 						session->incomingBuffer = cmd->private.change.item;
 						smlItemRef(session->incomingBuffer);
-						smlTrace(TRACE_INTERNAL, "New Buffer started. Buffer size: %i. Required: %i", xmlBufferLength(session->incomingBuffer->buffer), session->incomingBuffer->size);
+						smlTrace(TRACE_INTERNAL, "%s: New Buffer started. Buffer size: %i. Required: %i",
+							__func__,
+							xmlBufferLength(session->incomingBuffer->buffer),
+							session->incomingBuffer->size);
 						
 						char *bin = smlPrintBinary((char *) xmlBufferContent(session->incomingBuffer->buffer), xmlBufferLength(session->incomingBuffer->buffer));
-						smlTrace(TRACE_INTERNAL, "Content so far: %s\n", bin);
+						smlTrace(TRACE_INTERNAL, "%s: Content so far: %s\n",
+							__func__, bin);
 						g_free(bin);
 					}
 					
 					if (cmd->private.change.item->moreData == TRUE) {
-						smlTrace(TRACE_INTERNAL, "Got item with moreData %i", cmd->msgID);
+						smlTrace(TRACE_INTERNAL, "%s: Got item with moreData %i",
+							__func__, cmd->msgID);
 						
 						SmlStatus *reply = smlCommandNewReply(cmd, SML_CHUNK_ACCEPTED, error);
 						if (!reply)
@@ -872,7 +943,8 @@ parse_status:
 							smlSessionDispatchEvent(session, SML_SESSION_EVENT_CHILD_COMMAND, cmd, session->parentCommand, NULL, NULL);
 						}
 					} else {
-						smlTrace(TRACE_INTERNAL, "Replying with 407 since the session has ended2");
+						smlTrace(TRACE_INTERNAL, "%s: Replying with 407 since the session has ended2",
+							__func__);
 						SmlStatus *reply = smlCommandNewReply(cmd, SML_ERROR_AUTH_REQUIRED, error);
 						if (!reply)
 							goto error_free_cmd;
@@ -898,12 +970,16 @@ parse_status:
 		}
 	}
 	
-	/* Check that the message closes correctly */
+	/* Check that the message closes correctly
+	 * A final tag is only send if the SyncML package is finished.
+	 * A package can consists of several messages if MaxMsgSize
+	 * is smaller then the exchange data.
+	 */
 	SmlBool final = FALSE;
 	SmlBool end = FALSE;
 	if (!smlParserEnd(parser, &final, &end, error))
 		goto error_free_cmd;
-	
+
 	session->waiting = FALSE;
 
 	/* Dispatch the final event before we enter the following loop. Otherwise if upper
@@ -914,35 +990,104 @@ parse_status:
 	}
 	
 	/* We have to wait at this point until no status are pending anymore
-	 * if there is a incoming buffer for large objs open,
+	 * if there is an incoming buffer for large objs open,
 	 * the other side expects to see the status for the open buffer of course. so we have to
 	 * wait here for all open statuses, since they have to be added in the correct order and
 	 * the status for the open buffer is obviously last */	
 	while (1) {
-		unsigned int cmdRef, msgRef = 0;
+		unsigned int cmdRef, msgRef;
 		if (!smlAssemblerGetNextCmdRef(session->assembler, &cmdRef, &msgRef))
 			break;
-		
+		smlTrace(TRACE_INTERNAL, "%s: status found for cmd %d msg %d",
+			__func__, cmdRef, msgRef);
+
+		/* yes, we do some kind busy wait here */
 		usleep(10000);
 		
-		int i = 0;
+		/* if there is nothing in the queue (num == 0) then this means
+		 * that all status and commands are dispatched but the result of one or
+		 * more triggered activities is still not available
+		 * (therefore smlAssemblerGetNextCmdRef returns a command).
+		 *
+		 * smlQueueLengthPrio returns missing status count
+		 */
 		unsigned int num = smlQueueLengthPrio(session->command_queue);
-		smlTrace(TRACE_INTERNAL, "Now dispatching %i statuses. Session waiting: %i, prio: %i", num, session->waiting, smlQueueCheckPrio(session->command_queue));
+		smlTrace(TRACE_INTERNAL, "%s: Now dispatching %i statuses. Session waiting: %i, prio: %i",
+			__func__, num, session->waiting, smlQueueCheckPrio(session->command_queue));
+
+		/* If the session does not wait for the remote peer (session->waiting)
+		 * and there is a missing status in the queue
+		 * then the message is dispatched again.
+		 * The message is dispatched at maximimum once per missing status (i < num).
+		 * If smlSessionDispatch blocks because the dispatch_mutex is already in use 
+		 * and the situation changes (e.g. session->waiting is true)
+		 * then this is no problem because the function is safe against
+		 * mutliple calls (the SmlManager already requires this).
+		 */
+		int i = 0;
 		while (!session->waiting && smlQueueCheckPrio(session->command_queue) && i < num) {
 			smlSessionDispatch(session);
 			i++;
 		}
 	}
-	
-	smlTrace(TRACE_INTERNAL, "Final %i, waiting %i, sending %i", final, session->waiting, session->sending);
+
+	smlTrace(TRACE_INTERNAL, "%s: Final %i, waiting %i, sending %i",
+		__func__, final, session->waiting, session->sending);
 	if (final) {
+		/* Final means that the received message is the last message
+		 * of this SyncML package. When a final is received then the
+		 * library should not flush the session automatically because
+		 * the user of the library usually starts some special action.
+		 *
+		 * This can be for example a new package or some kind of
+		 * background action which results in informations expected by
+		 * the remote peer in the next message.
+		 * Example: OMA DS client (remote peer) sends a SYNC package
+		 *          and expects that the OMA DS server sends the server
+		 *          modifications at minimum after the last client
+		 *          modification.
+		 *
+		 * So again the library user must flush the session (e.g. handle
+		 * the SML_MANAGER_SESSION_FINAL event).
+		 */
 		if (session->pendingReplies) {
-			SmlPendingStatus *pending = (SmlPendingStatus *)session->pendingReplies->data;
-			smlErrorSet(error, SML_ERROR_GENERIC, "Didnt receive a reply for pending status (cmdID %i, msgID %i)", pending->cmdID, pending->msgID);
-			_smlSessionFreePendingReplies(session);
-			goto error;
+			/* If final is reached then there should be no
+			 * pendingReplies anymore. The problem is that some
+			 * commands (especially SYNC) results in new commands.
+			 * Nevertheless we have to check for old pendingReplies
+			 * and this we try to do here.
+			 *
+			 * Yes, this is some kind of bug detection.
+			 */
+			GList *o = NULL;
+        		for (o = session->pendingReplies; o; o = o->next) {
+                		SmlPendingStatus *pending = o->data;
+				/* This is an asynchronous library but the
+				 * message numbering and  sending is a serial
+				 * operation. So pendingReplies should only
+				 * exist for the last message. Anything else
+				 * should be a bug.
+				 */
+                		if (pending->msgID < session->lastMessageID)
+				{
+					smlErrorSet(error, SML_ERROR_GENERIC, "Didn't receive a reply for pending status (actual message %i, received message %i, cmdID %i, msgID %i)", session->lastMessageID, session->lastReceivedMessageID, pending->cmdID, pending->msgID);
+					goto error;
+				}
+                	}
 		}
-		
+
+		/* The incomingBuffer is used to store the last unhandled item.
+		 * The incoming buffer can be filled after a message is
+		 * completely handled if an object is too large for a message.
+		 * Usually this happens if you add images to your contacts ;)
+		 * The remote peer sends the moreData element in this case.
+		 *
+		 * Nevertheless if final is received then the package is
+		 * finished and so there cannot be any more data which is not
+		 * send by the client.
+		 *
+		 * Summary: there is a bug in this library or in the remote peer.
+		 */
 		if (session->incomingBuffer) {
 			smlErrorSet(error, SML_ERROR_GENERIC, "Incoming buffer was open when final arrived");
 			smlItemUnref(session->incomingBuffer);
@@ -950,13 +1095,29 @@ parse_status:
 			goto error;
 		}
 	} else if (!session->waiting && !session->sending) {
-		smlTrace(TRACE_INTERNAL, "This is not the final message. Empty? %i", smlAssemblerIsEmpty(session->assembler));
-		/* This was not the final message. So we need to either
-		 * send already added statuses or send an alert for the
-		 * next message */
+		/* If we don't wait already for an answer from the remote peer
+		 * (perhaps because a user of this library directly called
+		 * smlSessionFlush which causes a send operation of the
+		 * transport layer) and if we are waiting for or already
+		 * receiving a package (in terms of OMA DS specifications)
+		 * then we should send the collected stuff to the remote peer.
+		 */
+		smlTrace(TRACE_INTERNAL, "%s: This is not the final message. Empty? %i",
+			__func__, smlAssemblerIsEmpty(session->assembler));
+		/* The received message was not a final message. So this is
+		 * only a reaction to a part from a package. Sometimes such
+		 * replies only contain status but a message must include a
+		 * command.
+		 */
 		if (smlAssemblerIsEmpty(session->assembler)) {
-			/* The assembler is empty so we need to add an alert */
-			SmlCommand *alert = smlCommandNewAlert(SML_ALERT_NEXT_MESSAGE, session->source, session->target, NULL, NULL, NULL, error);
+			/* The assembler is empty. This means there is no
+			 * command included which is a requirement.
+			 * The solution is simple - an alert called next
+			 * message is added. this special alert does not need
+			 * a source or target because it only signals that the
+			 * remote peer should continue its work.
+			 */
+			SmlCommand *alert = smlCommandNewAlert(SML_ALERT_NEXT_MESSAGE, NULL, NULL, NULL, NULL, NULL, error);
 			if (!alert)
 				goto error_free_cmd;
 				
@@ -967,21 +1128,64 @@ parse_status:
 			
 			smlCommandUnref(alert);
 		}
+		/* Anything is fine and nobody else want to send a reply
+		 * to the received message. So the library flushs now.
+		 */
 		if (!_smlSessionFlushInternal(session, FALSE, error))
 			goto error_free_cmd;
 	}
 	
 	if (end) {
-		smlTrace(TRACE_INTERNAL, "Ending session now");
+		/* end means the last message does not contain a command which
+		 * is clearly illegal in SyncML except that the remote party
+		 * thinks that the device synchronization or management action
+		 * is successfully completed.
+		 *
+		 * The library can close the session now.
+		 */
+		smlTrace(TRACE_INTERNAL, "%s: Ending session now", __func__);
 		session->end = TRUE;
-		
+
+		/* If the OMA DS or DM session ends then a last flush should be
+		 * executed to signal the remote peer that the session end was
+		 * detected correctly.
+		 *
+		 * This only makes sense after a received final because
+		 * otherwise the session was already flushed in some way.
+		 */
 		if (session->sessionType != SML_SESSION_TYPE_SERVER) {
+			/* FIXME: This tolerates a bug in the HTTP client transport
+			 * FIXME: An empty message is send to the server to activate
+			 * FIXME: the libsoup client. This is clearly a hack!!!
+			 */
 			smlSessionDispatchEvent(session, SML_SESSION_EVENT_END, NULL, NULL, NULL, NULL);
 		} else {
-			/* if we are a server we have to flush here since we have to reply
-			 * with a header reply */
-			if (!_smlSessionFlushInternal(session, TRUE, error))
-				goto error;
+			/* This is the clean code which should be move one
+			 * block upward after the bug in the http client
+			 * transport layer is fixed.
+			 */
+			if (final)
+			{
+				smlTrace(TRACE_INTERNAL, "%s: Ending session now because this is a server",
+					__func__);
+				if (!_smlSessionFlushInternal(session, TRUE, error))
+					goto error;
+			} else {
+				if (!session->waiting)
+				{
+					/* The library does not send a message
+					 * after it received the last message
+					 * and neither the remote peer nor the
+					 * library send a final. This means
+					 * there is an active package. This is
+					 * an error of the library or the remote
+					 * peer.
+					 */
+					smlErrorSet(error, SML_ERROR_BAD_REQUEST,
+						"The message from the remote peer does not contain a command.");
+					goto error;
+				}
+			}
 		}
 	}
 
@@ -1056,6 +1260,24 @@ SmlLocation *smlSessionGetTarget(SmlSession *session)
 	return session->target;
 }
 
+void smlSessionSetCred(SmlSession *session, SmlCred *cred)
+{
+	smlAssert(session);
+	smlAssert(cred);
+	if (session->cred)
+		smlCredUnref(session->cred);
+	session->cred = cred;
+	if (cred->locName)
+		smlLocationSetName(session->source, cred->locName);
+	smlCredRef(session->cred);
+}
+
+SmlCred *smlSessionGetCred(SmlSession *session)
+{
+	smlAssert(session);
+	return session->cred;
+}
+
 /*@}*/
 
 /**
@@ -1100,8 +1322,10 @@ SmlSession *smlSessionNew(SmlSessionType sessionType, SmlMimeType mimetype, SmlP
 	SmlSession *session = smlTryMalloc0(sizeof(SmlSession), error);
 	if (!session)
 		goto error;
-	
+
+	session->dispatch_mutex = g_mutex_new();
 	session->refCount = 1;
+	session->eventCallback = NULL;
 	session->version = version;
 	session->protocol = protocol;
 	session->type = mimetype;
@@ -1125,6 +1349,8 @@ SmlSession *smlSessionNew(SmlSessionType sessionType, SmlMimeType mimetype, SmlP
 		
 	session->target = target;
 	smlLocationRef(target);
+
+	session->cred = NULL;
 	
 	/* Create the assembler that will parse messages send
 	 * from this session */
@@ -1178,6 +1404,9 @@ void smlSessionUnref(SmlSession *session)
 		
 		if (session->source)
 			smlLocationUnref(session->source);
+
+		if (session->cred)
+			smlCredUnref(session->cred);
 		
 		if (session->sessionID)
 			g_free(session->sessionID);
@@ -1200,6 +1429,8 @@ void smlSessionUnref(SmlSession *session)
 		
 		if (session->parentCommand)
 			smlCommandUnref(session->parentCommand);
+
+		g_mutex_free(session->dispatch_mutex);
 		
 		g_free(session);
 	}
@@ -1241,9 +1472,13 @@ SmlBool smlSessionFlush(SmlSession *session, SmlBool final, SmlError **error)
 	sesscmd->type = SML_SESSION_FLUSH;
 	sesscmd->final = final;
 	
-	smlTrace(TRACE_INTERNAL, "sending command %p", sesscmd);
+	smlTrace(TRACE_INTERNAL, "sending flush (%i) command %p via queue %p",
+		SML_SESSION_FLUSH, sesscmd, session->command_queue);
 
 	smlQueueSend(session->command_queue, sesscmd);
+
+	// session is still active even if the other side sent only status
+	session->end = FALSE;
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
@@ -1336,28 +1571,52 @@ void smlSessionUseOnlyReplace(SmlSession *session, SmlBool onlyReplace)
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
 
+/* FIXME: DEPRECATED */
 void smlSessionSetSendingLimit(SmlSession *session, unsigned int limit)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %u)", __func__, session, limit);
+	smlSessionSetSendingMaxMsgSize(session, limit);
+}
+
+/* FIXME: DEPRECATED */
+unsigned int smlSessionGetSendingLimit(SmlSession *session)
+{
+	return smlSessionGetSendingMaxMsgSize(session);
+}
+
+/* FIXME: DEPRECATED */
+void smlSessionSetReceivingLimit(SmlSession *session, unsigned int limit)
+{
+	smlSessionSetReceivingMaxMsgSize(session, limit);
+}
+
+void smlSessionSetSendingMaxMsgSize(SmlSession *session, unsigned int size)
+{
+	smlTrace(TRACE_ENTRY, "%s(%p, %u)", __func__, session, size);
 	smlAssert(session);
 	
-	smlAssemblerSetLimit(session->assembler, limit);
+	smlAssemblerSetLimit(session->assembler, size);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
 
-unsigned int smlSessionGetSendingLimit(SmlSession *session)
+unsigned int smlSessionGetSendingMaxMsgSize(SmlSession *session)
 {
 	smlAssert(session);
 	return smlAssemblerGetLimit(session->assembler);
 }
 
-void smlSessionSetReceivingLimit(SmlSession *session, unsigned int limit)
+void smlSessionSetReceivingMaxMsgSize(SmlSession *session, unsigned int size)
 {
-	smlTrace(TRACE_ENTRY, "%s(%p, %u)", __func__, session, limit);
+	smlTrace(TRACE_ENTRY, "%s(%p, %u)", __func__, session, size);
 	smlAssert(session);
-	
-	session->incomingMaxMsgSize = limit;
+
+	if (! smlSessionGetSendingMaxMsgSize(session) > 0 ||
+	    smlSessionGetSendingMaxMsgSize(session) > size)
+	{
+		session->incomingMaxMsgSize = size;
+	} else {
+		session->incomingMaxMsgSize = smlSessionGetSendingMaxMsgSize(session);
+	}
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
@@ -1432,7 +1691,11 @@ void smlSessionSetReceivingMaxObjSize(SmlSession *session, unsigned int limit)
 	smlTrace(TRACE_ENTRY, "%s(%p, %u)", __func__, session, limit);
 	smlAssert(session);
 	
-	session->receivingMaxObjSize = limit;
+	if (! smlSessionGetSendingMaxObjSize(session) > 0 ||
+	    smlSessionGetSendingMaxObjSize(session) > limit)
+	{
+		session->receivingMaxObjSize = limit;
+	}
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 }
