@@ -53,6 +53,14 @@ SmlBool smlTransportSend(SmlTransport *tsp, SmlLink *link, SmlTransportData *dat
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, tsp, link, data, error);
 	smlAssert(tsp);
 	smlAssert(data);
+
+	if (tsp->cached_error != NULL)
+	{
+		// the most parameters should be NULL because the cached error
+		// has nothing to do with the actual send request
+		smlTransportReceiveEvent(tsp, NULL, SML_TRANSPORT_EVENT_ERROR, NULL, NULL);
+		goto error;
+	}
 	
 	SmlTransportCommand *cmd = smlTryMalloc0(sizeof(SmlTransportCommand), error);
 	if (!cmd)
@@ -124,9 +132,27 @@ SmlBool smlTransportReceiveEvent(SmlTransport *tsp, SmlLink *link, SmlTransportE
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %i, %p, %p)", __func__, tsp, link, type, data, error);
 	smlAssert(tsp);
-	smlAssert(tsp->event_callback);
-	
-	SmlBool ret = tsp->event_callback(tsp, link, type, data, error, tsp->event_callback_userdata);
+
+	SmlBool ret = TRUE;
+	if (tsp->event_callback == NULL)
+	{
+		// we stop all communications here
+		smlTrace(TRACE_INTERNAL, "%s: init failed in transport protocol -> %s", __func__, smlErrorPrint(&error));
+		tsp->state = SML_TRANSPORT_ERROR;
+		tsp->cached_error = error;
+	} else {
+		if (tsp->cached_error != NULL)
+		{
+			smlTrace(TRACE_INTERNAL, "cached error detected - %s", __func__,
+				 smlErrorPrint(&tsp->cached_error));
+			// do not free the error here because you are not the owner of the error
+			error = tsp->cached_error;
+			ret = tsp->event_callback(tsp, link, type, data, error, tsp->event_callback_userdata);
+		} else {
+			smlTrace(TRACE_INTERNAL, "callback available", __func__);
+			ret = tsp->event_callback(tsp, link, type, data, error, tsp->event_callback_userdata);
+		}
+	}
 	
 	smlTrace(TRACE_EXIT, "%s: %i", __func__, ret);
 	return ret;
@@ -150,8 +176,13 @@ SmlBool smlTransportRunAsync(SmlTransport *tsp, SmlError **error)
 		smlErrorSet(error, SML_ERROR_GENERIC, "Transport was not in the state \"Uninitialized\"");
 		goto error;
 	}
-	
-	tsp->context = g_main_context_new();
+
+	// FIXME: bug in libsoup
+	// FIXME: asynchronous clients must use the default GMainContext
+	if (tsp->type != SML_TRANSPORT_HTTP_CLIENT)
+		tsp->context = g_main_context_new();
+	else
+		tsp->context = NULL;
 	
 	tsp->thread = smlThreadNew(tsp->context, error);
 	if (!tsp->thread)
@@ -260,6 +291,7 @@ SmlTransportData *smlTransportDataNew(char *data, unsigned long size, SmlMimeTyp
 	cmd->ownsData = ownsData;
 	cmd->refCount = 1;
 	cmd->needsAnswer = TRUE;
+	cmd->type_get = SML_MIMETYPE_UNKNOWN; 
 	
 	smlTrace(TRACE_EXIT, "%s: %p", __func__, cmd);
 	return cmd;
@@ -290,7 +322,7 @@ void smlTransportDataDeref(SmlTransportData *data)
 		return;
 	}
 	
-	if (data->ownsData)
+	if (data->ownsData && data->data != NULL)
 		g_free(data->data);
 	
 	g_free(data);
@@ -431,6 +463,9 @@ SmlTransport *smlTransportNew(SmlTransportType type, SmlError **error)
 	if (!tsp)
 		goto error;
 	tsp->type = type;
+	tsp->cached_error = NULL;
+	tsp->event_callback = NULL;
+	tsp->context = NULL;
 	
 	switch (type) {
 #ifdef ENABLE_HTTP
