@@ -22,11 +22,16 @@
 #define _HTTP_CLIENT_H_
 
 typedef struct SmlTransportHttpClientConfig {
-	uint16_t port;
+	int port;
 	char *url;
 	char *proxy;
+	char *username;
+	char *password;
+	char *cafile;
 } SmlTransportHttpClientConfig;
 
 SmlBool smlTransportHttpClientNew(SmlTransport *tsp, SmlError **error);
 
+SmlBool smlTransportHttpClientIsConnected(SmlTransport *tsp, SmlBool is_blocking, SmlError **error);
+
 #endif //_HTTP_CLIENT_H_
