@@ -67,10 +67,12 @@ static void _server_callback(SoupServerContext *context, SoupMessage *msg, gpoin
 	g_free(path);
 	
 	const char *header = soup_message_get_header(msg->request_headers, "Content-Type");
-	if (header && !g_strncasecmp(header, "application/vnd.syncml+xml", 26))
+	if (header && !g_strncasecmp(header, SML_ELEMENT_XML, strlen(SML_ELEMENT_XML)))
 		mimetype = SML_MIMETYPE_XML;
-	else if (header && !g_strncasecmp(header, "application/vnd.syncml+wbxml", 28))
+	else if(header && !g_strncasecmp(header, SML_ELEMENT_WBXML, strlen(SML_ELEMENT_WBXML)))
 		mimetype = SML_MIMETYPE_WBXML;
+	else if(header && !g_strncasecmp(header, SML_ELEMENT_SAN, strlen(SML_ELEMENT_SAN)))
+		mimetype = SML_MIMETYPE_SAN;
 	else if (header) {
 		smlErrorSet(&error, SML_ERROR_GENERIC, "Unknown mimetype");
 		soup_message_set_status (msg, SOUP_STATUS_BAD_REQUEST);
@@ -82,17 +84,28 @@ static void _server_callback(SoupServerContext *context, SoupMessage *msg, gpoin
 	}
 	
 	if (!msg->request.length) {
+		/* Tolerate empty message for now. Http client transport could send empty messages
+		   for initialization. Throwing an error here breaks testcase: manager_start_session 
+		   Empty Message for initialization got introduced with r251.
+		   It's not fixed yet if r251 will be the final implementation for http client initalization. */
+		soup_message_io_pause(msg);
+		smlTrace(TRACE_EXIT, "%s (empty message! tolerated for now ...)", __func__);
+		return;
+
+		/* FIXME: Throw an error again if changes in r251 got clarified/fixed. */
+		/*
 		smlErrorSet(&error, SML_ERROR_GENERIC, "No data sent");
 		soup_message_set_status (msg, SOUP_STATUS_BAD_REQUEST);
 		goto error;
+		*/
 	}
 	
 	SmlLink *link = smlLinkNew(env->tsp, msg, &error);
 	if (!link)
 		goto error;
-		
-	g_object_ref(msg);
-	
+
+	// FIXME: this is unnecessary because soup_message_io_pause ensure this alreay
+	// g_object_ref(msg);
 	
 	SmlTransportData *tspdata = smlTransportDataNew(msg->request.body, msg->request.length, mimetype, FALSE, &error);
 	if (!tspdata)
@@ -109,7 +122,8 @@ static void _server_callback(SoupServerContext *context, SoupMessage *msg, gpoin
 	return;
 
 error_unref_msg:
-	g_object_unref(msg);
+	// FIXME: this is unnecessary because libsoup's io_unpause makes the cleanup
+	// g_object_unref(msg);
 	smlLinkDeref(link);
 error:
 	soup_server_message_set_encoding (SOUP_SERVER_MESSAGE (msg), SOUP_TRANSFER_CONTENT_LENGTH);
@@ -148,14 +162,15 @@ static void *smlTransportHttpServerInit(SmlTransport *tsp, const void *data, Sml
 	}
 	env->port = config->port;
 	
-	//FIXME do url checking. use whitelist
+	// FIXME: do url checking. use whitelist
 	env->url = g_strdup(config->url);
 	env->interface = g_strdup(config->interface);
 	
 	smlTrace(TRACE_INTERNAL, "config: port %i, url %s, interface %s", config->port, config->url, config->interface);
 	
 	env->tsp = tsp;
-	
+
+	smlTrace(TRACE_INTERNAL, "http server uses context %p.", tsp->context);
 	if(!config->ssl_key || !config->ssl_crt) {
 		env->server = soup_server_new(SOUP_SERVER_ASYNC_CONTEXT, tsp->context, SOUP_SERVER_PORT, env->port, NULL);
 	} else {
@@ -191,7 +206,7 @@ static SmlBool smlTransportHttpServerFinalize(void *data, SmlError **error)
 	smlAssert(env->tsp);
 	
 	soup_server_quit(env->server);
-	g_object_unref(env->server);
+	// g_object_unref(env->server);
 	
 	g_free(env->url);
 	g_free(env->interface);
@@ -206,6 +221,7 @@ static void smlTransportHttpServerSend(void *userdata, void *linkdata, SmlTransp
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, userdata, linkdata, data, error);
 	smlAssert(userdata);
 	smlAssert(linkdata);
+	smlAssert(data);
 	SmlTransportHttpServerEnv *env = userdata;
 	smlAssert(env);
 	
@@ -213,30 +229,37 @@ static void smlTransportHttpServerSend(void *userdata, void *linkdata, SmlTransp
 	
 	if (error)
 		goto error_free_message;
-	
-	msg->response.owner = SOUP_BUFFER_USER_OWNED;
-	msg->response.length = data->size;
-	msg->response.body = data->data;
-	
+
+	// duplicate the data because sometimes data is freed to early
+	char *soupcopy = (char *) smlTryMalloc0(data->size, &error);
+	if (error)
+		goto error_free_message;
+	memcpy(soupcopy, data->data, data->size);
+
+	soup_message_set_status (msg, SOUP_STATUS_OK);
+	soup_server_message_set_encoding (SOUP_SERVER_MESSAGE(msg), SOUP_TRANSFER_CONTENT_LENGTH);
+
 	switch (data->type) {
 		case SML_MIMETYPE_XML:
-			soup_message_add_header(msg->response_headers, "Content-Type", "application/vnd.syncml+xml");
 			soup_message_add_header(msg->response_headers, "Accept", "application/vnd.syncml+xml");
+			soup_message_set_response (msg, "application/vnd.syncml+xml", SOUP_BUFFER_SYSTEM_OWNED,
+						soupcopy, data->size);
 			break;
 		case SML_MIMETYPE_WBXML:
-			soup_message_add_header(msg->response_headers, "Content-Type", "application/vnd.syncml+wbxml");
 			soup_message_add_header(msg->response_headers, "Accept", "application/vnd.syncml+wbxml");
+			soup_message_set_response (msg, "application/vnd.syncml+wbxml", SOUP_BUFFER_SYSTEM_OWNED,
+						soupcopy, data->size);
 			break;
 		default:
 			smlErrorSet(&error, SML_ERROR_GENERIC, "Unknown Mimetype");
 			goto error_free_message;
 	}
 	
-	soup_message_set_status (msg, SOUP_STATUS_OK);
-	soup_server_message_set_encoding (SOUP_SERVER_MESSAGE(msg), SOUP_TRANSFER_CONTENT_LENGTH);
 	soup_message_io_unpause(msg);
-	
-	g_object_unref(msg);
+
+	// FIXME: the documentation of libsoup never show an example with such a call
+	// FIXME: msg is perhaps in use by libsoup because this is asynchronous I/O
+	// g_object_unref(msg);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
 	return;
@@ -250,7 +273,7 @@ error_free_message:
 	soup_server_message_set_encoding (SOUP_SERVER_MESSAGE (msg), SOUP_TRANSFER_CONTENT_LENGTH);
 	soup_message_io_unpause(msg);
 
-	g_object_unref(msg);
+	// g_object_unref(msg);
 	smlErrorDeref(&error);
 	smlTrace(TRACE_EXIT, "%s: Sent Error", __func__);
 	return;
