@@ -57,6 +57,210 @@
  */
 /*@{*/
 
+static int obex_cable_at(SmlTransportObexClientEnv *userdata, char *cmd, char *rspbuf, int rspbuflen, int timeout)
+{
+  fd_set ttyset;
+  struct timeval tv;
+  gint fd;
+
+  char *answer = NULL;
+  char *answer_end = NULL;
+  unsigned int answer_size;
+
+  char tmpbuf[100] = {0,};
+  int actual;
+  int total = 0;
+  int done = 0;
+
+  fd = userdata->fd;
+  rspbuf[0] = 0;
+  if(fd < 0)
+    return -1;
+
+  if(cmd != NULL) {
+    // Write command
+    gint cmdlen;
+
+    cmdlen = strlen(cmd);
+    if(write(fd, cmd, cmdlen) < cmdlen) {
+      perror("Error writing to port");
+      return -1;
+    }
+  }
+
+  while(!done)  {
+    FD_ZERO(&ttyset);
+    FD_SET(fd, &ttyset);
+    tv.tv_sec = timeout;
+    tv.tv_usec = 0;
+    if(select(fd+1, &ttyset, NULL, NULL, &tv))  {
+      actual = read(fd, &tmpbuf[total], sizeof(tmpbuf) - total);
+      if(actual < 0)
+        return actual;
+      total += actual;
+
+      /* Answer didn't come within the length of the buffer. Cancel! */
+      if(total == sizeof(tmpbuf))
+        return -1;
+
+      if( (answer = index(tmpbuf, '\n')) )      {
+        // Remove first line (echo)
+        if( (answer_end = index(answer+1, '\n')) )      {
+          // Found end of answer
+          done = 1;
+        }
+      }
+    }
+    else        {
+      /* Anser didn't come in time. Cancel */
+      return -1;
+    }
+  }
+  //    printf("Answer: %s\n", answer);
+
+  // Remove heading and trailing \r
+  if((*answer_end == '\r') || (*answer_end == '\n'))
+    answer_end--;
+  if((*answer_end == '\r') || (*answer_end == '\n'))
+    answer_end--;
+  if((*answer == '\r') || (*answer == '\n'))
+    answer++;
+  if((*answer == '\r') || (*answer == '\n'))
+    answer++;
+
+  answer_size = (answer_end) - answer +1;
+
+  //    printf("Answer size=%d\n", answer_size);
+  if( (answer_size) >= rspbuflen )
+    return -1;
+
+
+  strncpy(rspbuf, answer, answer_size);
+  rspbuf[answer_size] = 0;
+  return 0;
+}
+
+static int smlTransportObexClientCableDisconnect(obex_t *handle, gpointer ud) {
+
+	SmlTransportObexClientEnv *userdata = (SmlTransportObexClientEnv *) ud;
+	if (userdata->fd >= 0) {
+		// Send a break to get out of OBEX-mode
+		if(ioctl(userdata->fd, TCSBRKP, 0) < 0) {
+			smlTrace(TRACE_INTERNAL, "Unable to send break!\n");
+		}
+
+		tcsetattr(userdata->fd, TCSANOW, &userdata->oldtio);
+		close(userdata->fd);
+	}
+
+	return 0;
+}
+
+static int smlTransportObexClientCableConnect(obex_t *handle, gpointer ud)
+{
+	SmlTransportObexClientEnv *userdata = (SmlTransportObexClientEnv *) ud;
+	struct termios newtio;
+	char rspbuf[200];
+
+	userdata->fd = open(userdata->path, O_RDWR|O_NONBLOCK|O_NOCTTY);
+	if (userdata->fd < 0)
+		return -1; 
+
+	tcgetattr(userdata->fd, &userdata->oldtio);
+	bzero(&newtio, sizeof(struct termios));
+	newtio.c_cflag = B115200 | CLOCAL | CS8 | CREAD | CRTSCTS;
+	newtio.c_cc[VMIN] = 1;
+	newtio.c_cc[VTIME] = 0;
+	newtio.c_iflag = IGNPAR; 
+	newtio.c_oflag = 0; 
+	tcflush(userdata->fd, TCIFLUSH);
+	tcsetattr(userdata->fd, TCSANOW, &newtio);
+
+	if (obex_cable_at(userdata, "ATZ\r", rspbuf, sizeof(rspbuf), 1) < 0) {
+		smlTrace(TRACE_INTERNAL, "Comm-error sending ATZ\n");
+		goto err;
+	}
+
+	if (strcasecmp("OK", rspbuf))        {
+		smlTrace(TRACE_INTERNAL, "Error doing ATZ (%s)\n", rspbuf);
+		goto err;
+	}
+
+
+	if (obex_cable_at(userdata, "AT+CPROT=0\r", rspbuf, sizeof(rspbuf), 1) < 0) {
+		smlTrace(TRACE_INTERNAL, "Comm-error sending AT+CPROT=0\n");
+		goto err;
+	}
+
+	if (strcasecmp("CONNECT", rspbuf))        {
+		smlTrace(TRACE_INTERNAL, "Error doing AT+CPROT=0 (%s)\n", rspbuf);
+		goto err;
+	}
+//	fcntl(userdata->fd, F_SETFL, O_NONBLOCK);
+	return 0;
+
+err:
+	smlTransportObexClientCableDisconnect(handle, userdata);
+	return -1;
+}
+
+static int smlTransportObexClientCableWrite(obex_t *handle, gpointer ud, guint8 *buf, int buflen) {
+
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %p, %i)", __func__, handle, ud, buf, buflen);
+	SmlTransportObexClientEnv *userdata = (SmlTransportObexClientEnv *) ud;
+	int written = 0;
+	int ret = 0;
+
+	while (ret >= 0 && written < buflen) {
+		ret = write(userdata->fd, buf + written, buflen - written);
+		if (ret >= 0)
+			written += ret;
+	}
+
+	smlTrace(TRACE_EXIT, "%s: %i", __func__, written);
+	return written;
+}
+
+gint obex_cable_handleinput(obex_t *handle, gpointer ud, gint timeout) {
+	smlTrace(TRACE_ENTRY, "%s(%p, %p, %i)", __func__, handle, ud, timeout);
+	struct timeval to;
+	fd_set readfds;
+	char buf[512];
+	SmlTransportObexClientEnv *userdata;
+	int ret = 0;
+	int actual = 0;
+
+	userdata = (SmlTransportObexClientEnv*) ud;
+	FD_ZERO(&readfds);
+	FD_SET(userdata->fd, &readfds);
+	to.tv_sec = timeout;
+	to.tv_usec = 0;
+
+
+	smlTrace(TRACE_INTERNAL, "%s - %i", __func__, __LINE__);
+	ret = select(userdata->fd + 1, &readfds, NULL, NULL, &to); 
+	
+	if (ret < 1)
+		goto error;
+	
+	smlTrace(TRACE_INTERNAL, "%s - %i", __func__, __LINE__);
+	if ((actual = read(userdata->fd, buf, sizeof(buf))) <= 0)
+		goto end;
+
+	smlTrace(TRACE_INTERNAL, "%s - %i", __func__, __LINE__);
+	OBEX_CustomDataFeed(handle, (unsigned char *) buf, actual);
+
+end:
+	smlTrace(TRACE_EXIT, "%s(%p, %p, %i)", __func__, handle, ud, timeout);
+	return actual;
+
+error:
+
+	smlTrace(TRACE_EXIT_ERROR, "%s: %i", __func__, ret);
+	return ret;
+
+}
+
 static void _smlObexEvent(obex_t *handle, obex_object_t *object, int mode, int event, int obex_cmd, int obex_rsp)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p, %i, %i, %i, %i)", __func__, handle, object, mode, event, obex_cmd, obex_rsp);
@@ -72,7 +276,7 @@ static void _smlObexEvent(obex_t *handle, obex_object_t *object, int mode, int e
 			env->busy = FALSE;
 			
 			if (obex_rsp != OBEX_RSP_SUCCESS) {
-				smlErrorSet(&error, SML_ERROR_GENERIC, "Request not successfull: %i", obex_rsp);
+				smlErrorSet(&error, SML_ERROR_IO_ERROR, "%s (0x%x)", OBEX_ResponseToString(obex_rsp), obex_rsp);
 				goto error;
 			}
 			
@@ -106,6 +310,7 @@ static void _smlObexEvent(obex_t *handle, obex_object_t *object, int mode, int e
 						goto error;
 					}
 					
+					smlTrace(TRACE_INTERNAL, "Got who: %s", who);
 					if (!who || strcmp(who, "SYNCML-SYNC")) {
 						smlErrorSet(&error, SML_ERROR_GENERIC, "Missing or wrong who response");
 						g_free(who);
@@ -171,8 +376,8 @@ static void _smlObexEvent(obex_t *handle, obex_object_t *object, int mode, int e
 			}
 			break;
 		case OBEX_EV_LINKERR:
-			smlTrace(TRACE_INTERNAL, "Link Error: %i", obex_rsp);
-			smlErrorSet(&error, SML_ERROR_GENERIC, "Link Error");
+			smlTrace(TRACE_INTERNAL, "Link Error: 0x%x", obex_rsp);
+			smlErrorSet(&error, SML_ERROR_GENERIC, "Link Error: 0x%x", obex_rsp);
 			goto error;
 			break;
 		case OBEX_EV_STREAMEMPTY:
@@ -229,7 +434,7 @@ static void *smlTransportObexClientInit(SmlTransport *tsp, const void *data, Sml
 			env->obexhandle = OBEX_Init(OBEX_TRANS_BLUETOOTH, _smlObexEvent, 0);
 			break;
 		case SML_OBEX_TYPE_SERIAL:
-			env->obexhandle = OBEX_Init(OBEX_TRANS_FD, _smlObexEvent, 0);			
+			env->obexhandle = OBEX_Init(OBEX_TRANS_CUST, _smlObexEvent, 0);	
 			break;
 		case SML_OBEX_TYPE_USB:
 			env->obexhandle = OBEX_Init(OBEX_TRANS_USB, _smlObexEvent, 0);
@@ -243,6 +448,18 @@ static void *smlTransportObexClientInit(SmlTransport *tsp, const void *data, Sml
 		smlErrorSet(error, SML_ERROR_GENERIC, "Unable to open connection");
 		goto error_free_env;
 	}
+
+	if (conf->type == SML_OBEX_TYPE_SERIAL) {
+		obex_ctrans_t cabletrans = { smlTransportObexClientCableConnect, 
+						smlTransportObexClientCableDisconnect,
+						NULL,
+						smlTransportObexClientCableWrite,
+						obex_cable_handleinput, smlTransportObexClientCableConnect };
+
+		cabletrans.customdata = env;
+
+		OBEX_RegisterCTransport(env->obexhandle, &cabletrans);
+	}
 	
 	OBEX_SetUserData(env->obexhandle, env);
 
@@ -264,6 +481,7 @@ error:
 	return NULL;
 }
 
+
 static SmlBool smlTransportObexClientFinalize(void *data, SmlError **error)
 {
 	smlTrace(TRACE_ENTRY, "%s(%p, %p)", __func__, data, error);
@@ -388,7 +606,17 @@ static void smlTransportObexClientConnect(void *data)
 	    
 	    /* Make the transmission raw (8 bit clean, no echo etc) */
 	    /* Set the speed to a higher value (9600 would be default) */
-	    cfmakeraw(&tio);
+
+#if defined(sun) && defined(__SVR4)
+	    tio.c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
+	    tio.c_oflag &= ~OPOST;
+	    tio.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
+	    tio.c_cflag &= ~(CSIZE|PARENB);
+	    tio.c_cflag |= CS8;
+#else	    
+	    cfmakeraw(&tio);	    
+#endif	    
+
 	    cfsetispeed(&tio, B115200);
 	    cfsetospeed(&tio, B115200);
 	
@@ -490,13 +718,30 @@ static void smlTransportObexClientDisconnect(void *data, void *linkdata)
 		goto error;
 	}
     
-	/* Lets see if it was successfull */
-	/*while (env->busy) {
-		if (OBEX_HandleInput(env->obexhandle, 20) < 0) {
+	/* Lets see if it was successfull. We wait a certain period of
+	 * time (3 sec) for the answer to the disconnect. If we dont receive
+	 * the answer we just disconnect anyways. Some phones never answer
+	 * with a A0 (success) to the disconnect request, so we must have
+	 * a timeout here */
+	int maxLoop = 3;
+	int i = 0;
+	while (env->busy) {
+		smlTrace(TRACE_INTERNAL, "Disconnect loop %i", i);
+		if (OBEX_HandleInput(env->obexhandle, 0) < 0) {
 			smlErrorSet(&error, SML_ERROR_GENERIC, "Unable to get answer");
 			goto error;
 		}
-	}*/
+		
+		if (i == maxLoop) {
+			smlTrace(TRACE_INTERNAL, "Did not receive a response to our disconnect");
+			break;
+		}
+		i++;
+		
+		if (env->busy) {
+			sleep(1);
+		}
+	}
 	smlTransportReceiveEvent(env->tsp, NULL, SML_TRANSPORT_EVENT_DISCONNECT_DONE, NULL, NULL);
 	
 	smlTrace(TRACE_EXIT, "%s", __func__);
@@ -608,7 +853,7 @@ static void smlTransportObexClientSend(void *userdata, void *link, SmlTransportD
 	/* Make a new get command */
 	obj = OBEX_ObjectNew(env->obexhandle, OBEX_CMD_GET);
 	if (!obj) {
-		smlErrorSet(&error, SML_ERROR_GENERIC, "Unable to create new put object");
+		smlErrorSet(&error, SML_ERROR_GENERIC, "Unable to create new get object");
 		goto error;
 	}
 
@@ -618,10 +863,41 @@ static void smlTransportObexClientSend(void *userdata, void *link, SmlTransportD
 	/* Add the connection id */
 	header.bq4 = env->connection_id;
 	OBEX_ObjectAddHeader(env->obexhandle, obj, OBEX_HDR_CONNECTION, header, sizeof(env->connection_id), OBEX_FL_FIT_ONE_PACKET);
-	
+
+	/* If smlTransportData.get_type is set, switch the mimetype for the GET command.
+	   This is needed for SyncML 1.2 SAN notificion:
+
+	   If this obex send sequence sends SyncML 1.2 SAN notification, switch the
+	   mimetype back to regular mimetype (not the SAN mimetype). The mimtype for the
+	   regular sync got stored in smlTransportData.type_get */
+
+	if (data->type_get != SML_MIMETYPE_UNKNOWN)
+	{
+		switch (data->type_get) {
+			case SML_MIMETYPE_WBXML:
+				target = SML_ELEMENT_WBXML;
+				break;
+			case SML_MIMETYPE_XML:
+				target = SML_ELEMENT_XML;
+				break;
+			case SML_MIMETYPE_SAN:
+				target = SML_ELEMENT_SAN;
+				break;
+			default:
+				smlErrorSet(&error, SML_ERROR_GENERIC, "Unknown mime type");
+				goto error_free_obj;
+		}
+
+		/* After this point, the entire conversion will be XML or WBXML. No more SAN mimetype. */
+		env->mimetype = data->type_get;
+
+		smlTrace(TRACE_INTERNAL, "Switch to new target: %s", target);
+	}
+
 	/* Now add the target mime type */
 	//header.bs = (unsigned char *)unicode;
 	//OBEX_ObjectAddHeader(env->obexhandle, obj, OBEX_HDR_TYPE, header, unicodesize, 0);
+
 	header.bs = (unsigned char *)target;
 	OBEX_ObjectAddHeader(env->obexhandle, obj, OBEX_HDR_TYPE, header, strlen(target) + 1, 0);
 	
