@@ -23,6 +23,9 @@
 
 #include <openobex/obex.h>
 
+#include <termios.h>
+#include <sys/ioctl.h>
+
 #define STREAM_CHUNK 4096
 
 typedef struct SmlTransportObexClientEnv {
@@ -33,7 +36,14 @@ typedef struct SmlTransportObexClientEnv {
 	uint32_t connection_id;
 	char *path;
 	unsigned int port;
-	
+
+	int fd;
+	int state;
+	char *databuf;
+	int *databuflen;
+	int connected;
+	struct termios oldtio;
+
 	SmlBool busy;
 	SmlBool error;
 	SmlMimeType mimetype;
