@@ -95,7 +95,7 @@ static void _manager_event(SmlManager *manager, SmlManagerEventType type, SmlSes
 			break;
 		case SML_MANAGER_CONNECT_DONE:
 		case SML_MANAGER_DISCONNECT_DONE:
-			fail();
+			fail(NULL);
 			break;
 		case SML_MANAGER_TRANSPORT_ERROR:
 			transport_errors++;
@@ -145,11 +145,16 @@ START_TEST (manager_receive)
 	clientConfig.port = 12000;
 	clientConfig.url = "http://127.0.0.1:12000";
 	clientConfig.proxy = NULL;
+	clientConfig.username = NULL;
+	clientConfig.password = NULL;
+	clientConfig.cafile = NULL;
 	
 	SmlTransportHttpServerConfig serverConfig;
 	serverConfig.port = 12000;
 	serverConfig.url = NULL;
 	serverConfig.interface = NULL;
+	serverConfig.ssl_key = NULL;
+	serverConfig.ssl_crt = NULL;
 	
 	managerTracker *clienttracker = g_malloc0(sizeof(managerTracker));
 	SmlManager *clientmanager = clienttracker->manager = smlManagerNew(client, &error);
@@ -227,11 +232,16 @@ START_TEST (manager_receive_session)
 	clientConfig.port = 12001;
 	clientConfig.url = "http://127.0.0.1:12001";
 	clientConfig.proxy = NULL;
+	clientConfig.username = NULL;
+	clientConfig.password = NULL;
+	clientConfig.cafile = NULL;
 	
 	SmlTransportHttpServerConfig serverConfig;
 	serverConfig.port = 12001;
 	serverConfig.url = NULL;
 	serverConfig.interface = NULL;
+	serverConfig.ssl_key = NULL;
+	serverConfig.ssl_crt = NULL;
 	
 	managerTracker *clienttracker = g_malloc0(sizeof(managerTracker));
 	SmlManager *clientmanager = clienttracker->manager = smlManagerNew(client, &error);
@@ -312,11 +322,16 @@ START_TEST (manager_end_session)
 	clientConfig.port = 12002;
 	clientConfig.url = "http://127.0.0.1:12002";
 	clientConfig.proxy = NULL;
+	clientConfig.username = NULL;
+	clientConfig.password = NULL;
+	clientConfig.cafile = NULL;
 	
 	SmlTransportHttpServerConfig serverConfig;
 	serverConfig.port = 12002;
 	serverConfig.url = NULL;
 	serverConfig.interface = NULL;
+	serverConfig.ssl_key = NULL;
+	serverConfig.ssl_crt = NULL;
 	
 	managerTracker *clienttracker = g_malloc0(sizeof(managerTracker));
 	SmlManager *clientmanager = clienttracker->manager = smlManagerNew(client, &error);
@@ -472,11 +487,17 @@ START_TEST (manager_start_session)
 	clientConfig.port = 12004;
 	clientConfig.url = "http://127.0.0.1:12004";
 	clientConfig.proxy = NULL;
+	clientConfig.username = NULL;
+	clientConfig.password = NULL;
+	clientConfig.cafile = NULL;
+
 	
 	SmlTransportHttpServerConfig serverConfig;
 	serverConfig.port = 12004;
 	serverConfig.url = NULL;
 	serverConfig.interface = NULL;
+	serverConfig.ssl_key = NULL;
+	serverConfig.ssl_crt = NULL;
 	
 	managerTracker *clienttracker = g_malloc0(sizeof(managerTracker));
 	SmlManager *clientmanager = clienttracker->manager = smlManagerNew(client, &error);
@@ -597,11 +618,16 @@ START_TEST (manager_register)
 	clientConfig.port = 12005;
 	clientConfig.url = "http://127.0.0.1:12005";
 	clientConfig.proxy = NULL;
+	clientConfig.username = NULL;
+	clientConfig.password = NULL;
+	clientConfig.cafile = NULL;
 	
 	SmlTransportHttpServerConfig serverConfig;
 	serverConfig.port = 12005;
 	serverConfig.url = NULL;
 	serverConfig.interface = NULL;
+	serverConfig.ssl_key = NULL;
+	serverConfig.ssl_crt = NULL;
 	
 	managerTracker *clienttracker = g_malloc0(sizeof(managerTracker));
 	SmlManager *clientmanager = clienttracker->manager = smlManagerNew(client, &error);
@@ -664,7 +690,7 @@ int main(void)
 	
 	SRunner *sr;
 	sr = srunner_create(s);
-	srunner_run_all(sr, CK_NORMAL);
+	srunner_run_all(sr, CK_VERBOSE);
 	nf = srunner_ntests_failed(sr);
 	srunner_free(sr);
 	return (nf == 0) ? 0 : 1;
