@@ -176,7 +176,7 @@ SmlBool _recv_server_event(SmlTransport *tsp, SmlLink *link, SmlTransportEventTy
 				smlTransportSetError(tsp, link, &newerror);
 				smlErrorDeref(&newerror);
 			} else
-				fail();
+				fail(NULL);
 			break;
 		case SML_TRANSPORT_EVENT_CONNECT_DONE:
 			g_atomic_int_inc(&client_connect_done);
@@ -966,7 +966,7 @@ int main(void)
 	
 	SRunner *sr;
 	sr = srunner_create(s);
-	srunner_run_all(sr, CK_NORMAL);
+	srunner_run_all(sr, CK_VERBOSE);
 	nf = srunner_ntests_failed(sr);
 	srunner_free(sr);
 	return (nf == 0) ? 0 : 1;
