@@ -2141,7 +2141,7 @@ int main(void)
 	
 	SRunner *sr;
 	sr = srunner_create(s);
-	srunner_run_all(sr, CK_NORMAL);
+	srunner_run_all(sr, CK_VERBOSE);
 	nf = srunner_ntests_failed(sr);
 	srunner_free(sr);
 	return (nf == 0) ? 0 : 1;
