@@ -882,6 +882,72 @@ START_TEST (check_assembler_devinf)
 }
 END_TEST
 
+START_TEST (check_assembler_devinf12)
+{
+	SmlError *error = NULL;
+	
+	SmlLocation *loc = smlLocationNew("/test", NULL, &error);
+	fail_unless(loc != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	SmlSession *session = smlSessionNew(SML_SESSION_TYPE_SERVER, SML_MIMETYPE_XML, SML_VERSION_12, SML_PROTOCOL_SYNCML, loc, loc, "1", 0, &error);
+
+	SmlAssembler *assm = start_assembler(session, 0);
+	fail_unless(assm != NULL, NULL);
+	
+	fail_unless(smlAssemblerReserveStatus(assm, 0, 1, 1, &error), NULL);
+	fail_unless(error == NULL, NULL);
+	
+	fail_unless(smlAssemblerAddHeader(assm, session, &error), NULL);
+	fail_unless(error == NULL, NULL);
+		
+	SmlStatus *status = smlStatusNew(SML_NO_ERROR, 0, 1, loc, loc, SML_COMMAND_TYPE_HEADER, &error);
+	fail_unless(status != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	smlLocationUnref(loc);
+	
+	fail_unless(smlAssemblerAddStatus(assm, status, &error) == SML_ASSEMBLER_RESULT_OK, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	smlStatusUnref(status);
+	
+	SmlDevInf *devinf = smlDevInfNew("mydevice", SML_DEVINF_DEVTYPE_WORKSTATION, &error);
+	fail_unless(devinf != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	SmlCommand *cmd = smlDevInfNewPut(devinf, SML_DEVINF_VERSION_12, &error);
+	fail_unless(cmd != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	cmd->cmdID = 1;
+	
+	smlDevInfUnref(devinf);
+	
+	fail_unless(smlAssemblerStartCommand(assm, NULL, cmd, &error) == SML_ASSEMBLER_RESULT_OK, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	smlCommandUnref(cmd);
+	
+	fail_unless(smlAssemblerEndCommand(assm, NULL, &error), NULL);
+	fail_unless(error == NULL, NULL);
+	
+	char *data = NULL;
+	unsigned int size = 0;
+	fail_unless(smlAssemblerRun(assm, &data, &size, NULL, FALSE, &error), NULL);
+	fail_unless(error == NULL, NULL);
+	fail_unless(data != NULL, NULL);
+	fail_unless(size != 0, NULL);
+	
+	g_free(data);
+	
+	fail_unless(smlAssemblerFlush(assm) == 1, NULL);
+	fail_unless(error == NULL, NULL);
+	
+	smlAssemblerFree(assm);
+	smlSessionUnref(session);
+}
+END_TEST
+
 START_TEST (check_assembler_order)
 {
 	SmlError *error = NULL;
@@ -1634,6 +1700,7 @@ Suite *assembler_suite(void)
 	create_case(s, "check_assembler_limit4", check_assembler_limit4);
 	create_case(s, "check_assembler_limit5", check_assembler_limit5);
 	create_case(s, "check_assembler_devinf", check_assembler_devinf);
+	create_case(s, "check_assembler_devinf12", check_assembler_devinf12);
 	create_case(s, "check_assembler_order", check_assembler_order);
 	create_case(s, "check_assembler_reuse", check_assembler_reuse);
 	create_case(s, "check_assembler_leak", check_assembler_leak);
@@ -1655,7 +1722,7 @@ int main(void)
 	
 	SRunner *sr;
 	sr = srunner_create(s);
-	srunner_run_all(sr, CK_NORMAL);
+	srunner_run_all(sr, CK_VERBOSE);
 	nf = srunner_ntests_failed(sr);
 	srunner_free(sr);
 	return (nf == 0) ? 0 : 1;
