@@ -24,5 +24,5 @@
 
 char *setup_testbed(char *fkt_name);
 void destroy_testbed(char *path);
-void create_case(Suite *s, const char *name, void (*function)(void));
+void create_case(Suite *s, const char *name, TFun function);
 SmlParser *start_parser(const char *data);
