@@ -269,7 +269,7 @@ static void _manager_event(SmlManager *manager, SmlManagerEventType type, SmlSes
 			sessionVersion = smlSessionGetVersion(session);
 
 			if (recvLimit)
-				smlSessionSetReceivingLimit(session, recvLimit);
+				smlSessionSetReceivingMaxMsgSize(session, recvLimit);
 
 			if (maxObjSize)
 				smlSessionSetReceivingMaxObjSize(session, maxObjSize);
