@@ -258,7 +258,7 @@ static void _manager_event(SmlManager *manager, SmlManagerEventType type, SmlSes
 			sessionVersion = smlSessionGetVersion(session);
 			
 			if (recvLimit)
-				smlSessionSetReceivingLimit(session, recvLimit);
+				smlSessionSetReceivingMaxMsgSize(session, recvLimit);
 				
 			if (maxObjSize)
 				smlSessionSetReceivingMaxObjSize(session, maxObjSize);
@@ -290,10 +290,10 @@ static void _manager_event(SmlManager *manager, SmlManagerEventType type, SmlSes
 			}
 			break;
 		case SML_MANAGER_SESSION_WARNING:
-			printf("WARNING: %s", smlErrorPrint(&error));
+			printf("WARNING: %s\n", smlErrorPrint(&error));
 			break;
 		case SML_MANAGER_SESSION_ERROR:
-			printf("There was an error in the session %s: %s", smlSessionGetSessionID(session), smlErrorPrint(&error));
+			printf("There was an error in the session %s: %s\n", smlSessionGetSessionID(session), smlErrorPrint(&error));
 			smlManagerQuit(manager);
 			g_main_loop_quit(loop);
 			break;
