@@ -61,6 +61,7 @@ static void usage (char *name, int ecode)
 	fprintf (stderr, "\t\t\t\t\t\"text/x-vcard\" for contacts, \"text/x-vcalendar\" for events, \"text/plain\" for\n");
 	fprintf (stderr, "\t\t\t\t\tnotes and \"text/x-vMessage\" for SMS\n");
 	fprintf (stderr, "\t\t\t\tPath is the local name of the database. You can choose something there\n\n");
+	fprintf (stderr, "[-s <device>]\t\tConnect to the serial device\n\n");
 	fprintf (stderr, "[-u <id>]\t\t\tConnect to the given usb interface number\n");
 	fprintf (stderr, "\t\t\t\tIf you dont specify an id, all available interface will be listed\n\n");
 	fprintf (stderr, "[-b <addr> <channel>]\t\tConnect to the given bluetooth device\n\n");
@@ -280,7 +281,7 @@ static void _manager_event(SmlManager *manager, SmlManagerEventType type, SmlSes
 			sessionVersion = smlSessionGetVersion(session);
 			
 			if (recvLimit)
-				smlSessionSetReceivingLimit(session, recvLimit);
+				smlSessionSetReceivingMaxMsgSize(session, recvLimit);
 				
 			if (maxObjSize)
 				smlSessionSetReceivingMaxObjSize(session, maxObjSize);
@@ -437,6 +438,8 @@ int main(int argc, char *argv[])
 	
 	SmlError *error = NULL;
 	char *identifier = NULL;
+	char *username = NULL;
+	char *password = NULL;
 	SmlNotificationVersion sanVersion = SML_SAN_VERSION_11;
 
 	if (!g_thread_supported ()) g_thread_init (NULL);
@@ -487,11 +490,30 @@ int main(int argc, char *argv[])
 			printf("Bluetooth not available in this build\n");
 			return 1;
 #endif
+		} else if (!strcmp (arg, "-s")) {
+			config.type = SML_OBEX_TYPE_SERIAL;
+
+			i++;
+			if (!argv[i])
+				usage (argv[0], 1);
+
+			config.url = g_strdup(argv[i]);
+
  		} else if (!strcmp (arg, "--identifier")) {
 			i++;
 			if (!argv[i])
 				usage (argv[0], 1);
 			identifier = g_strdup(argv[i]);
+		} else if (!strcmp (arg, "--username")) {
+			i++;
+			if (!argv[i])
+				usage (argv[0], 1);
+			username = g_strdup(argv[i]);
+		} else if (!strcmp (arg, "--password")) {
+			i++;
+			if (!argv[i])
+				usage (argv[0], 1);
+			password = g_strdup(argv[i]);
 		} else if (!strcmp (arg, "--recvLimit")) {
 			i++;
 			if (!argv[i])
@@ -549,7 +571,9 @@ int main(int argc, char *argv[])
 	SmlAuthenticator *auth = smlAuthNew(&error);
 	if (!auth)
 		goto error;
-	smlAuthSetEnable(auth, FALSE);
+
+	if (!username)
+		smlAuthSetEnable(auth, FALSE);
 	
 	if (!smlAuthRegister(auth, manager, &error))
 		goto error_free_auth;
